# -*- coding: utf-8 -*-
#
# Copyright 2020-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           plural, is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_model_args, \
    configure_input_fields, update_sample_parameters_args, \
    update_json_args, wait_for_available_tasks, get_basic_seed

from bigmler.resourcesapi.common import SEED, FIELDS_QS, \
    ALL_FIELDS_QS, EVALUATE_SAMPLE_RATE


def set_linear_regression_args(args, name=None, fields=None,
                               objective_id=None,
                               linear_regression_fields=None):
    """Return linear regression arguments dict

    """
    if name is None:
        name = args.name
    if linear_regression_fields is None:
        linear_regression_fields = args.linear_regression_fields_
    if objective_id is None:
        objective_id = args.objective_id_

    linear_regression_args = set_basic_model_args(args, name)
    linear_regression_args.update({
        "seed": SEED if args.seed is None else args.seed
    })

    if objective_id is not None and fields is not None:
        linear_regression_args.update({"objective_field": objective_id})
    if linear_regression_fields and fields is not None:
        input_fields = configure_input_fields(fields, linear_regression_fields)
        linear_regression_args.update(input_fields=input_fields)
    if ((args.evaluate and args.test_split == 0 and args.test_datasets is None)
            or args.cross_validation_rate > 0):
        linear_regression_args.update(seed=SEED)
        if args.cross_validation_rate > 0:
            args.sample_rate = 1 - args.cross_validation_rate
            args.replacement = False
        elif (args.sample_rate == 1 and args.test_datasets is None
              and not args.dataset_off):
            args.sample_rate = EVALUATE_SAMPLE_RATE
    linear_regression_args.update({"sample_rate": args.sample_rate})
    linear_regression_args.update({"bias": args.bias})
    if args.field_codings is not None:
        linear_regression_args.update(\
            {"field_codings": args.field_codings_})

    linear_regression_args = update_sample_parameters_args( \
        linear_regression_args, args)

    if 'linear_regression' in args.json_args:
        update_json_args(linear_regression_args,
                         args.json_args.get('linear_regression'),
                         fields)
    return linear_regression_args


def create_linear_regressions(datasets, linear_regression_ids,
                              linear_regression_args,
                              args, api=None, path=None,
                              session_file=None, log=None):
    """Create remote linear regressions

    """
    if api is None:
        api = bigml.api.BigML()

    linear_regressions = linear_regression_ids[:]
    existing_linear_regressions = len(linear_regressions)
    linear_regression_args_list = []
    datasets = datasets[existing_linear_regressions:]
    # if resuming and all linear regressions were created,
    # there will be no datasets left
    if datasets:
        if isinstance(linear_regression_args, list):
            linear_regression_args_list = linear_regression_args

        # Only one linear regression per command, at present
        number_of_linear_regressions = 1
        message = dated("Creating %s.\n" %
                        plural("linear regression",
                               number_of_linear_regressions))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_linear_regressions):
            wait_for_available_tasks(inprogress,
                                     args.max_parallel_linear_regressions,
                                     api, "linearregression")
            if linear_regression_args_list:
                linear_regression_args = linear_regression_args_list[i]
            if args.cross_validation_rate > 0:
                new_seed = get_basic_seed(i + existing_linear_regressions)
                linear_regression_args.update(seed=new_seed)

            if (args.test_datasets and args.evaluate):
                dataset = datasets[i]
                linear_regression = api.create_linear_regression( \
                    dataset, linear_regression_args, retries=None)
            elif args.dataset_off and args.evaluate:
                multi_dataset = args.test_dataset_ids[:]
                del multi_dataset[i + existing_linear_regressions]
                linear_regression = api.create_linear_regression( \
                    multi_dataset, linear_regression_args, retries=None)
            else:
                linear_regression = api.create_linear_regression( \
                datasets, linear_regression_args, retries=None)
            linear_regression_id = check_resource_error( \
                linear_regression, "Failed to create linear regression: ")
            log_message("%s\n" % linear_regression_id, log_file=log)
            linear_regression_ids.append(linear_regression_id)
            inprogress.append(linear_regression_id)
            linear_regressions.append(linear_regression)
            log_created_resources("linear_regressions",
                                  path,
                                  linear_regression_id, mode='a')

        if args.verbosity:
            if bigml.api.get_status(linear_regression)['code'] != \
                    bigml.api.FINISHED:
                try:
                    linear_regression = check_resource( \
                        linear_regression, api.get_linear_regression,
                        query_string=query_string,
                        raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished linear regression:"
                             " %s" %
                             str(exception))
                linear_regressions[0] = linear_regression
            message = dated("linear regression created: %s\n" %
                            get_url(linear_regression))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, linear_regression)

    return linear_regressions, linear_regression_ids


def get_linear_regressions(linear_regression_ids,
                           args, api=None, session_file=None):
    """Retrieves remote linear regression in its actual status

    """
    if api is None:
        api = bigml.api.BigML()

    linear_regression_id = ""
    linear_regressions = linear_regression_ids
    linear_regression_id = linear_regression_ids[0]
    message = dated("Retrieving %s. %s\n" %
                    (plural("linear regression", len(linear_regression_ids)),
                     get_url(linear_regression_id)))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one linear regression to predict at present
    try:
        # we need the whole fields structure when exporting fields
        query_string = FIELDS_QS if not args.export_fields else ALL_FIELDS_QS
        linear_regression = check_resource(linear_regression_ids[0],
                                           api.get_linear_regression,
                                           query_string=query_string,
                                           raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished linear regression: %s" % \
            str(exception))
    linear_regressions[0] = linear_regression

    return linear_regressions, linear_regression_ids


def set_publish_linear_regression_args(args):
    """Set args to publish linear regression

    """
    public_linear_regression = {}
    if args.public_linear_regression:
        public_linear_regression = {"private": False}
        if args.model_price:
            public_linear_regression.update(price=args.model_price)
        if args.cpp:
            public_linear_regression.update(credits_per_prediction=args.cpp)
    return public_linear_regression


def update_linear_regression(linear_regression, linear_regression_args,
                             args, api=None, path=None, session_file=None):
    """Updates linear regression properties

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Updating linear regression. %s\n" %
                    get_url(linear_regression))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    linear_regression = api.update_linear_regression(linear_regression, \
        linear_regression_args)
    check_resource_error(linear_regression,
                         "Failed to update linear regression: %s"
                         % linear_regression['resource'])
    linear_regression = check_resource(linear_regression,
                                       api.get_linear_regression,
                                       query_string=FIELDS_QS,
                                       raise_on_error=True)
    if is_shared(linear_regression):
        message = dated("Shared linear regression link. %s\n" %
                        get_url(linear_regression, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, linear_regression)

    return linear_regression



# -*- coding: utf-8 -*-
#
# Copyright 2021-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - image annotations formatting

"""

import os
import json
import sys
import xml.etree.ElementTree as ET
import re
import glob
import csv


from zipfile import ZipFile

import cv2

from bigml.constants import IMAGE_EXTENSIONS
from bigml.util import infer_field_type

import bigmler.utils as u




FILE_ATTR = "file"
BBOXES_ATTR = "boxes"


def relative_path(base_dir, absolute_path):
    """Uses the basename of the file if no base directory is given,
    and the relative path otherwise

    """
    if base_dir is None:
        return os.path.basename(absolute_path)
    return os.path.relpath(absolute_path, base_dir)


def get_file_ext(filename):
    """Getting the file extension in lowercase and without the dot """
    return os.path.splitext(filename)[1].lower()[1:]


def fields_from_annotations(annotations_file):
    """Infers the type of the fields that will contain the annotations
    in an annotations file.
    """
    if not (annotations_file and os.path.exists(annotations_file)):
        sys.exit("Failed to find the annotations file.")
    with open(annotations_file) as annotations_handler:
        annotations = json.load(annotations_handler)
        annotations_fields = {}
        for annotation in annotations:
            if "file" in annotation:
                for field, value in annotation.items():
                    if field != "file" and field not in annotations_fields:
                        annotations_fields[field] = infer_field_type(
                            field, value)
    return list(annotations_fields.values())


def labels_from_annotations(annotations_file):
    """Returns the labels in each field. """
    if not (annotations_file and os.path.exists(annotations_file)):
        sys.exit("Failed to find the annotations file.")
    with open(annotations_file) as annotations_handler:
        annotations = json.load(annotations_handler)
        annotations_labels = {}
        for annotation in annotations:
            if "file" in annotation:
                for field, value in annotation.items():
                    if field != "file":
                        if field not in annotations_labels:
                            annotations_labels[field] = []
                        if isinstance(value, dict):
                            if value["label"] not in annotations_labels[field]:
                                annotations_labels[field].append(
                                    value["label"])
                        elif isinstance(value, str):
                            if value not in \
                                    annotations_labels[field]:
                                annotations_labels[field].append(value)
                        elif isinstance(value, list):
                            for item in value:
                                if item["label"] not in \
                                        annotations_labels[field]:
                                    annotations_labels[field].append(
                                        item["label"])
    return annotations_labels


def bigml_metadata(args, images_list=None, new_fields=None):
    """Creates a metadata file to summarize the locations of images and
    annotations
    """
    if images_list is None:
        images_list = []
    image_names = [os.path.basename(image_path) for image_path in images_list]
    try:
        if args.images_file is None and args.images_dir and \
                os.path.exists(args.images_dir):
            try:
                zipname = args.images_dir.split(os.path.sep)[-1]
                if not zipname:
                    zipname = args.images_dir.split(os.path.sep)[-2]
            except Exception:
                zipname = "annotated_images"
            zip_path = os.path.join(args.output_dir, "%s.zip" % zipname)
            files = glob.glob(os.path.join(args.images_dir, "**"),
                              recursive=True)
            images_list = [filename for
                           filename in files if get_file_ext(filename)
                           in IMAGE_EXTENSIONS and (
                           (not args.annotated_only) or
                           os.path.basename(filename) in image_names)]

        #pylint: disable=locally-disabled,possibly-used-before-assignment
        if images_list:
            if os.path.exists(zip_path):
                os.remove(zip_path)
            with ZipFile(zip_path, 'w') as zip_obj:
                for filename in images_list:
                    zip_obj.write(filename,
                                 relative_path(args.images_dir, filename))
            args.images_file = zip_path

        output = args.images_file
        # if annotations are also uploaded
        if args.annotations_file:
            if new_fields is None:
                new_fields = fields_from_annotations(args.annotations_file)

            metafile_name = os.path.join(args.output_dir, "metadata.json")
            annotations_file = os.path.relpath(args.annotations_file,
                args.output_dir)
            images_file = os.path.relpath(args.images_file,
                args.output_dir)

            with open(metafile_name, "w") as \
                    metafile:
                meta_info = {"description": ("Transformed file for images"
                                             " annotations"),
                             "images_file": images_file,
                             "source_id": args.source,
                             "new_fields": new_fields,
                             "annotations": annotations_file}
                json.dump(meta_info, metafile)
            output = metafile_name
    except Exception as exc:
        sys.exit(f"Failed to create an annotated images upload file: {exc}")

    return output


def bigml_coco_file(args, session_file):
    """Translates from alternative annotations format, like VOC, YOLO or
    MSCOCO to the format accepted by BigML

    """
    if args.annotations_file is not None:
        args.original_annotations_file = args.annotations_file
    args.annotations_file = os.path.join(args.output_dir, "annotations.json")
    filenames = []
    if args.annotations_language == "VOC":
        filenames = voc_to_cocojson(args.annotations_dir, args, session_file)
    elif args.annotations_language == "YOLO":
        filenames = yolo_to_cocojson(args.annotations_dir, args, session_file)
    elif args.annotations_language == "COCO":
        filenames = mscoco_to_cocojson(args.original_annotations_file,
                                       args, session_file)
    elif args.annotations_language == "CSV":
        filenames = csv_to_cocojson(args.original_annotations_file, args,
                                    session_file)
    if args.annotations_field is None:
        args.annotations_field = BBOXES_ATTR

    return bigml_metadata(args, images_list=filenames,
                          new_fields=[{"name": args.annotations_field,
                                       "optype": "regions"}])


def get_image_info(annotation_root):
    """Returns the basic image descriptors"""
    filename = annotation_root.findtext('filename')
    path = annotation_root.findtext('path')
    if filename is None and path is not None:
        filename = os.path.basename(path)
    img_name = os.path.basename(filename)
    img_id = os.path.splitext(img_name)[0]
    if isinstance(img_id, str):
        img_id = int(re.findall(r'\d+', img_id)[0])

    size = annotation_root.find('size')
    width = int(size.findtext('width'))
    height = int(size.findtext('height'))

    image_info = {
        'filename': filename,
        'height': height,
        'width': width,
        'id': img_id
    }
    return image_info


def get_coco_annotation_from_object(obj, filename, logfile, warnings):
    """Returns the annotation properties reading them from coco notation"""
    label = obj.findtext('name')
    bndbox = obj.find('bndbox')
    ## xmin = int(bndbox.findtext('xmin')) - 1
    xmin = int(bndbox.findtext('xmin'))
    ## ymin = int(bndbox.findtext('ymin')) - 1
    ymin = int(bndbox.findtext('ymin'))
    xmax = int(bndbox.findtext('xmax'))
    ymax = int(bndbox.findtext('ymax'))

    ## assert xmax >= xmin and ymax >= ymin, \
    ## f"Box size error !: (xmin, ymin, xmax, ymax): {xmin, ymin, xmax, ymax}"
    ## o_width = xmax - xmin
    ## o_height = ymax - ymin

    # logging possible errors
    if xmin == xmax or ymin == ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      "  min equal to max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1
    if xmin > xmax or ymin > ymax:
        logfile.write(f"Possible bndbox error in {filename}:\n" +
                      "  min greater than max: (xmin, ymin, xmax, ymax):" +
                      f"  {xmin, ymin, xmax, ymax}\n")
        warnings += 1

    annotation = {
        'label': label,
        'xmin': xmin,
        'ymin': ymin,
        'xmax': xmax,
        'ymax': ymax
    }
    return annotation, warnings


def yolo_to_cocojson(yolo_dir, args, session_file):
    """Builds annotations file from yolo separated annotation files """
    ## assume .txt extension for yolo annotation files,
    ## may be input as args
    yolo_extension = '.txt'
    ## extensions of acceptable image files

    output_json_array = []

    filenames = []
    if args.annotations_field is None:
        args.annotations_field = BBOXES_ATTR

    logfile_name = args.annotations_file + ".log"
    with open(logfile_name, "w") as logfile:

        warnings = 0
        message = "Start converting YOLO files in " + yolo_dir + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n%s" % message)

        # Images will be found either in the images_dir file or where
        # the yolo annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            images_dir = args.images_dir
        else:
            images_dir = yolo_dir
        filenames = glob.glob(os.path.join(images_dir, "**"),
                              recursive=True)
        filenames = [os.path.abspath(filename) for
                     filename in filenames if get_file_ext(filename) in
                     IMAGE_EXTENSIONS]

        ## Read yolo annotation txt file
        yolo_file_list = []
        for a_file in glob.glob(os.path.join(yolo_dir, "**"), recursive=True):
            if a_file.endswith(yolo_extension):
                # print(os.path.join(yolo_dir, a_file))
                logfile.write(a_file + "\n")
                yolo_file_list.append(os.path.abspath(a_file))

        logfile.write("\n")
        ## Check if the yolo files (.txt) in the yolo_dir have their
        ## corresponding images, .jpg, .jpeg, .png, etc
        for yolo_filename in yolo_file_list:
            filename_base = os.path.splitext(yolo_filename)[0]
            # print("filename_base: " + filename_base)
            # print(yolo_dir + "/" + filename_base + ".*")
            image_filename = ''

            matched_files = [filename for filename in
                    glob.glob(filename_base + ".*") if not
                    filename.endswith(yolo_extension)]
            if len(matched_files) == 0:
                logfile.write(" Warning: no image file for " +
                        yolo_filename +"\n")
            else:
                # print(*matched_files, sep = ", ")
                # print("matched_files length: " + str(len(matched_files)))

                ## traverse the matched file lists to find the image file,
                ## giving a warning each time a matched file is not in the
                ## image_extensions list.
                ## if there are more than one supported image file,
                ## the last one in the matched_file list is used
                for a_file in matched_files:
                    filenames.append(a_file)
                    ext = get_file_ext(a_file)
                    if ext in IMAGE_EXTENSIONS:
                        image_filename = a_file
                    else:
                        warnings += 1
                        logfile.write("Warning: unknown image file for " +
                                      yolo_filename + "\n")
                        logfile.write(a_file)
                        logfile.write("\n")

            if not image_filename or image_filename.isspace():
                continue

            # print("image_filename: " + image_filename + "\n")
            logfile.write("converting for: " + image_filename + "\n")
            image_filename_base = relative_path(images_dir, image_filename)

            #pylint: disable=locally-disabled,no-member
            image_file = cv2.imread(image_filename)
            if image_file is not None:
                if image_filename not in filenames:
                    filenames.append(image_filename)
                with open(yolo_filename, "r") as yolo_file:
                    yolo_read_lines = yolo_file.readlines()

                logfile.write("taking as filename: " +
                              image_filename_base + "\n")

                one_image_dict = {
                    ## possible args options for full path or basename
                    FILE_ATTR: image_filename_base,
                    args.annotations_field: []
                }

                ## yolo format - (label, xc, yc, width, height)
                ## coco format - (label, xmin, ymin, xmax, ymax)
                for yolo_line in yolo_read_lines:
                    one_yolo_annotation = yolo_line
                    label = one_yolo_annotation.split()[0]

                    x_center = float(one_yolo_annotation.split()[1])
                    y_center = float(one_yolo_annotation.split()[2])
                    width = float(one_yolo_annotation.split()[3])
                    height = float(one_yolo_annotation.split()[4])

                    if x_center.is_integer() and y_center.is_integer() and \
                            width.is_integer() and height.is_integer():
                        round_x_center = x_center
                        round_y_center = y_center
                        proper_width = width
                        proper_height = height
                    else:
                        float_x_center = image_file.shape[1] * x_center
                        float_y_center = image_file.shape[0] * y_center
                        float_width = image_file.shape[1] * width
                        float_height = image_file.shape[0] * height

                        int_x_center = int(float_x_center)
                        int_y_center = int(float_y_center)
                        int_width = int(float_width)
                        int_height = int(float_height)

                        round_width = round(float_width)
                        round_height = round(float_height)

                        ## Yolo uses normalized coordinates to specify
                        ## the bonding boxes. [x_center, y_center, width, height]
                        ## Because they are normalized, i.e. ratios to the width
                        ## and height of the image, they stay the same when the image
                        ## is enlarged or shrank. But the bounding boxes may
                        ## lose pixels due to rounding.

                        ## The following operations try to reverse the effect of
                        ## rounding and add a pixel to the bouding box if reasonable.
                        x_center_decimal = float_x_center - int(float_x_center)
                        y_center_decimal = float_y_center - int(float_y_center)
                        #print("xc_decimal=", x_center_decimal, " yc_decimal=",
                        #        y_center_decimal, "\n")
                        logfile.write("xc_decimal=" + str(x_center_decimal) +
                                      " yc_decimal=" + str(y_center_decimal) +
                                      "\n")
                        if x_center_decimal <= 0.25:
                            round_x_center = int(float_x_center)
                        elif 0.75 >= x_center_decimal > 0.25:
                            round_x_center = int(float_x_center) + 0.5
                        else:
                            round_x_center = int(float_x_center) + 1

                        if y_center_decimal <= 0.25:
                            round_y_center = int(float_y_center)
                        elif 0.75 >= y_center_decimal > 0.25:
                            round_y_center = int(float_y_center) + 0.5
                        else:
                            round_y_center = int(float_y_center) + 1

                        if round_x_center - int(round_x_center) == 0:
                            if (round(float_width) % 2) == 0:
                                proper_width = round(float_width)
                            else:
                                proper_width = round(float_width) + 1
                        else:
                            if (round(float_width) % 2) == 0:
                                proper_width = round(float_width) + 1
                            else:
                                proper_width = round(float_width)

                        if round_y_center - int(round_y_center) == 0:
                            if (round(float_height) % 2) == 0:
                                proper_height = round(float_height)
                            else:
                                proper_height = round(float_height) + 1
                        else:
                            if (round(float_height) % 2) == 0:
                                proper_height = round(float_height) + 1
                            else:
                                proper_height = round(float_height)


                        #print("xc=", x_center, " yc=", y_center, " w=", width,
                        #      " h=", height, "\n")
                        logfile.write("xc=" + str(x_center) + " yc=" + str(y_center) +
                                    " w=" + str(width) + " h=" + str(height) + "\n")
                        #print("float_xc=", float_x_center, " float_yc=", float_y_center,
                        #      " float_w=", float_width, " float_h=", float_height, "\n")
                        logfile.write("float_xc=" + str(float_x_center) + " float_yc=" +
                                      str(float_y_center) + " float_w=" +
                                      str(float_width) + " float_h=" +
                                      str(float_height) + "\n")
                        #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                        #      " round_w=", round_width, " round_h=", round_height, "\n")
                        logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                                      str(round_y_center) + " round_w=" +
                                      str(round_width) + " round_h=" +
                                      str(round_height) + "\n")
                        #print("round_xc=", round_x_center, " round_yc=", round_y_center,
                        #    " proper_w=", proper_width, " proper_h=", proper_height, "\n")
                        logfile.write("round_xc=" + str(round_x_center) + " round_yc=" +
                                      str(round_y_center) + " proper_w=" +
                                      str(proper_width) + " proper_h=" +
                                      str(proper_height) + "\n")
                        #print("int_xc=", int_x_center, " int_yc=", int_y_center,
                        #      " int_w=", int_width, " int_h=", int_height, "\n\n")
                        logfile.write("int_xc=" + str(int_x_center) + " int_yc=" +
                                      str(int_y_center) + " int_w=" + str(int_width) +
                                      " int_h=" + str(int_height) + "\n\n")

                    ## x_min = int_x_center - int_width/2
                    x_min = int(round_x_center - proper_width/2)
                    ## y_min = int_y_center - int_height/2
                    y_min = int(round_y_center - proper_height/2)
                    ## x_max = x_min + int_width
                    x_max = int(x_min + proper_width)
                    ## y_max = y_min + int_height
                    y_max = int(y_min + proper_height)
                    annotation = {
                            'label': label,
                            'xmin': x_min,
                            'ymin': y_min,
                            'xmax': x_max,
                            'ymax': y_max
                            }
                    one_image_dict[args.annotations_field].append(annotation)

                # output_json_dict['annotations'].append(one_image_dict)
                output_json_array.append(one_image_dict)
            else:
                warnings += 1
                logfile.write("Warning: failed to find image file for " +
                              image_file + "\n")
                logfile.write("\n")


    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                  f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    with open(args.annotations_file, 'w') as handler:
        json.dump(output_json_array, handler, indent=2)

    return [relative_path(images_dir, filename) for filename in \
        filenames]


def voc_to_cocojson(voc_dir, args, session_file):
    """Translates annotations from a VOC format, where each image is associated
    to a .xml file that contains one object per associated info. It returns
    the list of images it refers to.

    """

    output_json_array = []

    filenames = []

    annotation_file_list = []
    if args.annotations_field is None:
        args.annotations_field = BBOXES_ATTR

    for file in os.listdir(voc_dir):
        if file.endswith(".xml"):
            annotation_file_list.append(os.path.join(voc_dir, file))
    logfile_name = args.annotations_file + ".log"
    with open(logfile_name, "w") as logfile:

        ## Start bounding_box_id, possible future use, may be input as args
        bndbox_id = 1
        warnings = 0
        message = "Start converting VOC files in " + voc_dir + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n\n%s\n" % message)

        # Images will be found either in the images_dir file or where
        # the yolo annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            filenames = glob.glob(os.path.join(args.images_dir, "**"),
                                  recursive=True)
            filenames = [os.path.abspath(filename) for
                         filename in filenames if get_file_ext(filename) in
                         IMAGE_EXTENSIONS]

        for a_file in annotation_file_list:
            ## Read annotation xml per image
            annotation_tree = ET.parse(a_file)
            annotation_root = annotation_tree.getroot()

            img_info = get_image_info(annotation_root)
            folder = annotation_root.findtext('folder') or ''
            base_dir = os.getcwd()
            if args.images_dir is not None:
                base_dir = os.path.join(base_dir, args.images_dir)
            image_filename_base = os.path.join(folder, img_info['filename'])
            filename = image_filename_base
            if args.images_dir is not None:
                filename = os.path.join(base_dir, image_filename_base)

            if os.path.exists(filename):
                if filename not in filenames:
                    filenames.append(filename)

                ## possible args options for full path or basename
                logfile.write("converting for: " + filename + "\n")
                logfile.write("taking as filename: " + image_filename_base + "\n")
                
                one_image_dict = {
                    ## possible args options for full path or basename
                    FILE_ATTR: image_filename_base,
                    args.annotations_field: []
                }

                for obj in annotation_root.findall('object'):
                    annotation, warnings = get_coco_annotation_from_object( \
                        obj, filename, logfile, warnings)
                    one_image_dict[args.annotations_field].append(annotation)
                    bndbox_id = bndbox_id + 1

                output_json_array.append(one_image_dict)
            else:
                warnings += 1
                logfile.write("failed to find: " + filename + "\n")

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    with open(args.annotations_file, 'w') as handler:
        json.dump(output_json_array, handler, indent=2)

    return [relative_path(args.images_dir, filename) for filename in \
        filenames]

def mscoco_to_cocojson(mscoco_file, args, session_file):
    """Translates annotations from a MS COCO format, where each image is
    associated with a JSON file that contains one object per associated info.
    Maps images, categories and annotations to image file names, labels and
    regions. It returns the list of images it refers to.

    """

    output_json_array = []

    filenames = []
    labels = {}
    images = {}

    logfile_name = args.annotations_file + ".log"

    with open(logfile_name, "w") as logfile:

        warnings = 0
        message = "Start converting COCO file from " + mscoco_file + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n\n%s\n" % message)

        # Loading the MS-COCO json into memory

        with open(mscoco_file, "r") as handle:
            data = json.load(handle)

        # Images will be found either in the images_dir file or where
        # the annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            filenames = glob.glob(os.path.join(args.images_dir, "**"),
                                  recursive=True)
            paths = [os.path.abspath(filename) for
                     filename in filenames if get_file_ext(filename) in
                     IMAGE_EXTENSIONS]
            filenames = [os.path.basename(path) for path in paths]

        if args.annotations_field is None:
            args.annotations_field = BBOXES_ATTR

        # Extracting the file_name and id into a dict
        images = dict([image['id'],
                       { FILE_ATTR: image['file_name'],
                         args.annotations_field: [] }]
                      for image in data['images'] if image['file_name'] in
                      filenames)
        if data.get("categories") and data['categories'][0].get("name"):
            # Extract the image category labels into a dict
            labels = dict([category['id'],
                           { "name": category['name'],
                             "super": category.get('supercategory', "") } ]
                          for category in data['categories'])
        # Adding the regions data
        if data.get('annotations'):
            for annotation in data['annotations']:
                images[annotation["image_id"]][args.annotations_field].append({
                    "label": labels[annotation['category_id']]['name'],
                    "xmin": int(annotation["bbox"][0]),
                    "ymin": int(annotation["bbox"][1]),
                    "xmax": int(annotation["bbox"][0] + annotation["bbox"][2]),
                    "ymax": int(annotation["bbox"][1] + annotation["bbox"][3])
                })

                if labels[annotation['category_id']]['super']:
                    images[annotation["image_id"]][
                        args.annotations_field].append({
                        "label": labels[annotation['category_id']]['super'],
                        "xmin": int(annotation["bbox"][0]),
                        "ymin": int(annotation["bbox"][1]),
                        "xmax": int(annotation["bbox"][0] +
                                    annotation["bbox"][2]),
                        "ymax": int(annotation["bbox"][1] +
                                    annotation["bbox"][3])
                    })

        output_json_array = list(images.values())

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    filenames = [image['file'] for image in output_json_array]

    with open(args.annotations_file, 'w') as handler:
        json.dump(output_json_array, handler, indent=2)

    return [relative_path(args.images_dir, filename) for filename in \
        filenames]


def transform_regions(data, regions_field):
    """Escaping quotes in regions labels as exported in a CSV from a dataset
    and setting the expected attributes.
    """

    annotations = {}
    for record in data:
        regions = record[regions_field]
        filename = record["filename"]
        if regions != "":
            annotations[filename] = []
            boxes = []
            # we keep the compact format, but scape quotes
            regions = re.sub(r'""', '\\"', regions)
            annotations[filename] = {FILE_ATTR: filename,
                                     regions_field: regions}
    return annotations


def csv_to_cocojson(csv_file, args, session_file):
    """Translates annotations from the CSV downloaded from am images dataset.
    Maps image file names and regions information to image file names,
    labels and boxes. It returns the list of images it refers to.

    """

    output_json_array = []

    filenames = []
    labels = {}
    images = {}

    logfile_name = args.annotations_file + ".log"

    with open(logfile_name, "w") as logfile:

        warnings = 0
        message = "Start converting CSV file from " + csv_file + "\n"
        u.log_message(message, session_file, console=args.verbosity)
        logfile.write("\n\n%s\n" % message)

        # Loading the CSV file as a list into memory

        with open(csv_file, "r") as handler:
            csv_reader = csv.DictReader(handler)
            data = [row for row in csv_reader]

        # Images will be found either in the images_dir file or where
        # the annotation file points to
        if args.images_dir is not None and os.path.exists(args.images_dir):
            filenames = glob.glob(os.path.join(args.images_dir, "**"),
                                  recursive=True)
            paths = [os.path.abspath(filename) for
                     filename in filenames if get_file_ext(filename) in
                     IMAGE_EXTENSIONS]
            sizes = []
            filenames = [os.path.relpath(path, start=args.images_dir)
                         for path in paths]

        annotations = transform_regions(data, args.annotations_field)

        annotated_images = list(annotations.keys())
        annotation_boxes = list(annotations.values())

        for index, image in enumerate(annotated_images):
            image = image.replace("/", os.path.sep)
            annotated_images[index] = image
            if image not in filenames:
                sys.exit(f"Failed to find the annotated file {image} in"
                         f" {args.images_dir}.")

    if warnings > 0:
        message = f"\nThere are {warnings} warnings, " \
                f"see the log file {logfile_name}\n"
        u.log_message(message, session_file, console=args.verbosity)

    with open(args.annotations_file, 'w') as handler:
        json.dump(annotation_boxes, handler, indent=2)

    return [relative_path(args.images_dir, filename) for filename in \
        annotated_images]



# -*- coding: utf-8 -*-
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer source processing

"""

def get_source_options(defaults=None):
    """source-related options

    """

    if defaults is None:
        defaults = {}

    options = {
        # Path to the training set.
        '--train': {
            "action": 'store',
            "dest": 'training_set',
            'nargs': '?',
            "default": defaults.get('train', None),
            "help": "Training set path."},

        # Path to data
        '--data': {
            "action": 'store',
            "dest": 'training_set',
            'nargs': '?',
            "default": defaults.get('training_set', None),
            "help": "Path to source data."},

        # If a BigML source is provided, the script won't create a new one
        '--source': {
            "action": 'store',
            "dest": 'source',
            "default": defaults.get('source', None),
            "help": "BigML source Id."},

        # If a BigML json file containing a source structure is provided,
        # the script will use it.
        '--source-file': {
            'action': 'store',
            'dest': 'source_file',
            'default': defaults.get('source_file', None),
            'help': "BigML source JSON structure file."},

        # If a sources file is provided, the source ID will be extracted from
        # it
        '--source-in': {
            "action": 'store',
            "dest": 'source_in',
            "default": defaults.get('source_in', None),
            'help': ("Path to a file containing source/ids (one per line)"
                     "that will be used as the source ID to work on.")},

        # If a sources file is provided, the source IDs will be extracted from
        # it
        '--sources-in': {
            "action": 'store',
            "dest": 'sources_in',
            "default": defaults.get('sources_in', None),
            'help': ("Path to a file containing many source/ids (one per line)"
                     ". Used as the list of sources for a composite "
                     " source.")},

        # The path to a file containing names if you want to alter BigML's
        # default field names or the ones provided by the train file header.
        # Kept for backwards compatibility
        '--field-names': {
            'action': 'store',
            'dest': 'field_attributes',
            'default': defaults.get('field_names', None),
            'help': ("Path to a csv file describing field names. One"
                     " definition per line (e.g., 0,'Last Name').")},

        # The path to a file containing attributes if you want to alter BigML's
        # default field attributes or the ones provided by the train file
        # header.
        '--field-attributes': {
            'action': 'store',
            'dest': 'field_attributes',
            'default': defaults.get('field_attributes', None),
            'help': ("Path to a csv file describing field attributes."
                     " One definition per line"
                     " (e.g., 0,'Last Name').")},
        # The path to a file containing types if you want to alter BigML's
        # type auto-detect.
        '--types': {
            'action': 'store',
            'dest': 'types',
            'default': defaults.get('types', None),
            'help': ("Path to a file describing field types. One"
                     " definition per line (e.g., 0, 'numeric').")},
        # Set when the training set file doesn't include a header on the first
        # line.
        '--no-train-header': {
            'action': 'store_false',
            'dest': 'train_header',
            'default': defaults.get('train_header', True),
            'help': "The train set file hasn't a header."},

        # Set when the training set file does include a header on the first
        # line. (opposed to --no-train-header)
        '--train-header': {
            'action': 'store_true',
            'dest': 'train_header',
            'default': defaults.get('train_header', True),
            'help': "The train set file has a header."},

        # Locale settings.
        '--locale': {
            'action': 'store',
            'dest': 'user_locale',
            'default': defaults.get('locale', 'en_US.UTF8'),
            'help': "Chosen locale code string."},

        # The path to a file containing source attributes.
        '--source-attributes': {
            'action': 'store',
            'dest': 'source_attributes',
            'default': defaults.get('source_attributes', None),
            'help': ("Path to a json file describing source"
                     " attributes.")},

        # Training set field separator. Defaults to the locale csv
        # separator.
        '--training-separator': {
            'action': 'store',
            'dest': 'training_separator',
            'default': defaults.get('training_separator', None),
            'help': ("Training set field separator.")},

        # Name of the project to be created and/or used in resource creation
        '--project': {
            'action': 'store',
            'dest': 'project',
            'default': defaults.get('project', None),
            'help': ("Name of the project to be created and/or used in"
                     "resource creation.")},

        # Id of the project to be used in resource creation
        '--project-id': {
            'action': 'store',
            'dest': 'project_id',
            'default': defaults.get('project_id', None),
            'help': ("Id of the project to be used in "
                     "resource creation.")},

        # The path to a file containing the fields information available in
        # the resource
        '--export-fields': {
            'action': 'store',
            'dest': 'export_fields',
            'default': defaults.get('export_fields', None),
            'help': ("Path to a csv file describing the available field"
                     " information. The first row is used as header.")},

        # The path to a file containing field attributes that will be used
        # to modify the fields structure.
        '--import-fields': {
            'action': 'store',
            'dest': 'import_fields',
            'default': defaults.get('import_fields', None),
            'help': ("Path to a csv file describing field attributes."
                     " The first row is used as header and rows are"
                     " expected to comply the the --export-fields output.")},

        # Close the source for editing
        '--close': {
            'action': 'store_true',
            'dest': 'closed',
            'default': defaults.get('closed', None),
            'help': ("Close the source (no editing allowed).")},

        # Check if the source is editable. Clone the source into an editable
        # one otherwise.
        '--open': {
            'action': 'store_false',
            'dest': 'closed',
            'default': defaults.get('closed', None),
            'help': ("Whether or not the source will be open for editing.")},

        # A comma-separated list of source identifiers to remove from a
        # composite and, if they do not belong to any other composite, delete
        # as individual resources.
        '--delete-sources': {
            'action': 'store',
            'dest': 'delete_sources',
            'default': defaults.get('delete_sources', None),
            'help': ("A comma-separated list of source identifiers to remove"
                     " from a composite and, if they do not belong to any"
                     " other composite, delete as individual resources.")},

        # A comma-separated list of source identifiers to remove from a
        # composite and, if they do not belong to any other composite, delete
        # as individual resources.
        '--remove-sources': {
            'action': 'store',
            'dest': 'remove_sources',
            'default': defaults.get('remove_sources', None),
            'help': ("A comma-separated list of source identifiers to remove"
                     " from a composite.")},

        # A comma-separated list of source identifiers to remove from a
        # composite and, if they do not belong to any other composite, delete
        # as individual resources.
        '--sources': {
            'action': 'store',
            'dest': 'sources',
            'default': defaults.get('sources', None),
            'help': ("A comma-separated list of source identifiers to replace"
                     " the existing ones in the composite.")},

        # A comma-separated list of source identifiers to add to the
        # composite.
        '--add-sources': {
            'action': 'store',
            'dest': 'add_sources',
            'default': defaults.get('add_sources', None),
            'help': ("A comma-separated list of source identifiers to add"
                     " to the composite.")},

        # Values to be replaced in some fields and rows.
        '--row-values-json': {
            'action': 'store',
            'dest': 'row_values_json',
            'default': defaults.get('row_values_json', None),
            'help': ("Path to a JSON file that contains an array of objects"
                     " specifying values for different rows and label fields "
                     " in a composite via the following properties: "
                     " `indices, components, value, field."
                     " E.g.: [{\"field\": \"field1\", \"indices\": [0, 3],"
                     " \"value\": 8}, {\"field\": \"field2\","
                     " \"value\": 4}]."
                     " In the second element, no indices or components are "
                     " specified, so the properties \"row_components\" and "
                     " \"row_indices\" will be used as default for those.")},

        # Default for the components property of row_values
        '--row-components': {
            'action': 'store',
            'dest': 'row_components',
            'default': defaults.get('row_components', None),
            'help': ("Comma-separated list of IDs corresponding to the "
                     "sources in a composite that will be updated by "
                     "default when using \"row_values\" with no \"components\""
                     " specification.")},

        # Default for the indices property of row_values
        '--row-indices': {
            'action': 'store',
            'dest': 'row_indices',
            'default': defaults.get('row_indices', None),
            'help': ("Comma-separated list of indices corresponding to the "
                     "sources in a composite that will be updated by "
                     "default when using \"row_values\" with no \"indices\""
                     " specification.")},

        # Image analysis options: dimensions
        '--dimensions': {
            'action': 'store_true',
            'dest': 'dimensions',
            'default': defaults.get('dimensions', False),
            'help': "Whether to use dimensions as image analysis features"},

        # Image analysis options: average pixels
        '--average-pixels': {
            'action': 'store_true',
            'dest': 'average_pixels',
            'default': defaults.get('average_pixels', False),
            'help': ("Whether to use average pixels as image analysis"
                     "features. Captures basic color information.")},

        # Image analysis options: level histogram
        '--level-histogram': {
            'action': 'store_true',
            'dest': 'level_histogram',
            'default': defaults.get('level_histogram', False),
            'help': ("Whether to use level histogram as image analysis"
                     " features. Captures detailed color information.")},

        # Image analysis options: Histogram of Gradients
        '--HOG': {
            'action': 'store_true',
            'dest': 'hog',
            'default': defaults.get('hog', False),
            'help': ("Whether to use Histogram of Gradients as image analysis"
                     " features. Captures profiles.")},

        # Image analysis options: wavelet subbands
        '--ws-level': {
            'action': 'store',
            'type': int,
            'dest': 'ws_level',
            'default': defaults.get('ws_level', None),
            'help': ("If set, wavelet subbands are used as image analysis"
                     " features and the number decides the level "
                     " decomposition used. Captures textures.")},

        # Image analysis options: pretrained CNNs
        '--pretrained-cnn': {
            'action': 'store',
            'dest': 'pretrained_cnn',
            'default': defaults.get('pretrained_cnn', None),
            'choices': ["mobilenet", "mobilenet2", "resnet18"],
            'help': ("If set, pretrained CNNs output nodes are used as"
                     " image analysis features.")},

        # Image analysis options
        '--no-image-analysis': {
            'action': 'store_false',
            'dest': 'image_analysis',
            'default': defaults.get('image_analysis', None),
            'help': "Avoiding image analysis feature generation."},

        # Annotations language
        # syntax used to handle image annotations
        '--annotations-language': {
            'action': 'store',
            'dest': 'annotations_language',
            'default': defaults.get('annotations_language', None),
            'choices': ["VOC", "YOLO", "COCO", "CSV"],
            'help': ("Language used to provide the annotations for images."
                     "Annotations are expected to be provided using "
                     "one file per image. The --train option should point"
                     " to the directory that contains both images and"
                     " the corresponding annotations, unless some "
                     " folder attribute is provided in each"
                     " annotation. In that case it should point to"
                     " the folder's' parent directory and --anotations-dir"
                     " should be used to point to the annotations files.")},

        # Annotations file
        # File that contains annotations for images
        '--annotations-file': {
            'action': 'store',
            'dest': 'annotations_file',
            'default': defaults.get('annotations_file', None),
            'help': "File that contains the annotations to images."},

        # Annotations directory
        # For annotations stored in individual files, like VOC or YOLO,
        # directory where the annotation files are located
        '--annotations-dir': {
            'action': 'store',
            'dest': 'annotations_dir',
            'default': defaults.get('annotations_dir', None),
            'help': ("Directory for individual annotation files."
                     " Used when annotations are provided using "
                     "one file per image. The --train option should point"
                     " to the directory that contains both images and"
                     " the corresponding annotations, unless some "
                     " folder attribute is provided in each"
                     " annotation. In that case it should point to"
                     " the folder's' parent directory and --anotations-dir"
                     " should be used to point to the annotations files.")},

        # Only images that have annotations are included in the composite
        '--annotated-only': {
            'action': 'store_true',
            'dest': 'annotated_only',
            'default': defaults.get('annotated_only', False),
            'help': ("Whether to add only the annotated images.")},

        # Annotations field name
        # Name of the field that will contain the annotations
        '--annotations-field': {
            'action': 'store',
            'dest': 'annotations_field',
            'default': defaults.get('annotations_field', None),
            'help': "Name of the field that will contain the annotations."},


        # Images file
        # Compressed file with images used as reference for annotations
        '--images-file': {
            'action': 'store',
            'dest': 'images_file',
            'default': defaults.get('images_file', None),
            'help': ("Compressed file with images used as reference for "
                     "annotations.")}}

    return options



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

try:
    import simplejson as json
except ImportError:
    import json

from bigml.api import check_resource

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_, eq_, approx_


def given_i_create_bigml_resources_using_source_to_evaluate(
    step, output=None):
    """Step: I create BigML resources using source to evaluate and log
    evaluation in <output>
    """
    ok_(output is not None)
    command = ("bigmler --evaluate --source " + world.source['resource'] +
               " --output " + output)
    shell_execute(command, output)


def read_id_from_file(path):
    """Reading the id in first line in a file

    """
    with open(path) as id_file:
        return id_file.readline().strip()


def given_i_create_bigml_resources_using_dataset_to_evaluate(
    step, output=None):
    """Creating a default evaluation from dataset"""
    ok_(output is not None)
    command = ("bigmler --evaluate --dataset " + world.dataset['resource'] +
               " --output " + output)
    shell_execute(command, output)


def i_create_all_resources_to_evaluate_with_model_and_map(
    step, data=None, fields_map=None, output=None):
    """Step: I create BigML resources using test file <data> and a fields
    map <fields_map> to evaluate a model and log evaluation in <output>
    """
    ok_(data is not None and fields_map is not None and output is not None)
    command = ("bigmler --evaluate --test " + res_filename(data) +
               " --model " +
               world.model['resource'] + " --output " + output +
               " --fields-map " + res_filename(fields_map))
    shell_execute(command, output)


def i_create_all_resources_to_evaluate_with_model(
    step, data=None, output=None):
    """Step: I create BigML resources using test file <data> to evaluate
    a model and log evaluation in <output>
    """
    ok_(data is not None and output is not None)
    command = ("bigmler --evaluate --test " + res_filename(data) +
               " --model " +
               world.model['resource'] + " --output " + output)
    shell_execute(command, output)


def given_i_create_bigml_resources_using_dataset_to_evaluate_with_model(
    step, output=None):
    """Step: I create BigML resources using a dataset to evaluate a
    model and log evaluation in <output>
    """
    ok_(output is not None)
    command = ("bigmler --evaluate --dataset " +
               world.dataset['resource']  + " --model " +
               world.model['resource'] + " --output " + output)
    shell_execute(command, output)


def i_create_with_split_to_evaluate(step, data=None, split=None, output=None):
    """Step: I create BigML resources uploading train <data> file to evaluate
    with test-split <split> and log evaluation in <output>
    """
    ok_(data is not None and split is not None and output is not None)
    command = ("bigmler --evaluate --train " + res_filename(data) +
               " --test-split " + split +  " --output " + output)
    shell_execute(command, output)


def i_create_proportional_to_evaluate(step, test=None):
    """Step: I evaluate <test> with proportional missing strategy"""
    ok_(test is not None)
    test = res_filename(test)
    output_dir = world.directory + "_eval/"
    output = output_dir + os.path.basename(world.output)
    command = ("bigmler --evaluate --model " +
               world.model['resource'] +
               " --test " + test +
               " --missing-strategy proportional --output " +
               output)
    shell_execute(command, output)


def then_the_evaluation_file_is_like(step, check_file_json):
    """Step: the evaluation file is like <check_file_json>"""
    check_file_path = res_filename(check_file_json)
    evaluation_file_json = world.output + ".json"
    try:
        with open(evaluation_file_json) as eval_handler:
            with open(check_file_path) as check_handler:
                check = check_handler.readline()
                evaluation = eval_handler.readline()
                check = json.loads(check)
                evaluation = json.loads(evaluation)
        if 'model' in check:
            for metric, value in check['model'].items():
                if not isinstance(value, list) and not isinstance(value, dict):
                    if isinstance(value,str):
                        eq(check['model'][metric],
                           evaluation['model'][metric],
                           msg=f"model {metric}")
                        eq_(check['mode'][metric],
                            evaluation['mode'][metric],
                            msg=f"mode {metric}")
                    else:
                        approx_(check['model'][metric],
                                evaluation['model'][metric],
                                msg=f"model {metric}")
                        approx_(check['mode'][metric],
                                evaluation['mode'][metric],
                                msg=f"mode {metric}")
        else:
            del check["datasets"]
            del evaluation["datasets"]
            eq_(check, evaluation)
    except Exception as exc:
        with open("%s.new" % check_file_path, "w") as eval_handler:
            json.dump(evaluation, eval_handler)
        ok_(False, msg=str(exc))


def i_check_create_dataset(step, dataset_type=None):
    """Step: I check that the <dataset_type> dataset has been created"""
    dataset_file = os.path.join(world.directory, "dataset_%s" % dataset_type)
    try:
        with open(dataset_file) as handler:
            dataset = check_resource(handler.readline().strip(),
                                     world.api.get_dataset)
        world.datasets.append(dataset['resource'])
        world.dataset = dataset
    except Exception as exc:
        ok_(False, msg=str(exc))


def i_check_evaluation_key(step, key=None, value=None):
    """Step: the evaluation key <key> value for the model is greater than
    <value>
    """
    evaluation_file_json = world.output + ".json"
    try:
        with open(evaluation_file_json) as handler:
            evaluation = handler.readline()
            evaluation = json.loads(evaluation)
            model_evaluation = evaluation['model']
            ok_(model_evaluation[key] > float(value),
                "model key: %s, expected >: %s" % ( \
                model_evaluation[key], float(value)))
    except Exception as exc:
        ok_(False, msg=str(exc))


def i_create_with_split_to_evaluate_ensemble(
    step, data=None, number_of_models=None, split=None, output=None):
    """Step: I create BigML resources uploading train <data> file to evaluate
    an ensemble of <number_of_models> models with test-split <split> and log
    evaluation in <output>
    """
    ok_(data is not None and split is not None and output is not None)
    command = ("bigmler --evaluate --train " + res_filename(data) +
               " --test-split " + split +
               " --number-of-models " + number_of_models +
               " --output " + output)
    shell_execute(command, output)


def i_evaluate_ensemble_with_dataset(
    step, ensemble_dir=None, dataset_dir=None, output=None):
    """Step: I evaluate the ensemble in directory <ensemble_dir> with the
    dataset in directory <dataset_dir> and log evaluation in <output>
    """
    ok_(ensemble_dir is not None and dataset_dir is not None and
        output is not None)
    ensemble_id = read_id_from_file(os.path.join(ensemble_dir, "ensembles"))
    dataset_id = read_id_from_file(os.path.join(dataset_dir, "dataset_test"))
    command = ("bigmler --dataset " + dataset_id +
               " --ensemble " + ensemble_id + " --store" +
               " --output " + output + " --evaluate")
    shell_execute(command, output)


def i_evaluate_ensemble_with_dataset_and_options( \
    step, ensemble_dir=None, dataset_dir=None, output=None, options=None):
    """Step: I evaluate the ensemble in directory <ensemble_dir> with the
    dataset in directory <dataset_dir> and log evaluation in <output>
    """
    ok_(ensemble_dir is not None and dataset_dir is not None and
        output is not None and options is not None)
    ensemble_id = read_id_from_file(os.path.join(ensemble_dir, "ensembles"))
    dataset_id = read_id_from_file(os.path.join(dataset_dir, "dataset_test"))
    command = ("bigmler --dataset " + dataset_id +
               " --ensemble " + ensemble_id + " --store" +
               " --output " + output + " --evaluate " + options)
    shell_execute(command, output)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for tableau

This module defines functions that generate tableau code to make local
predictions
"""

from bigml.tree_utils import PYTHON_OPERATOR, COMPOSED_FIELDS

from bigml.predict_utils.common import mintree_split, get_predicate, get_node, \
    missing_branch, none_value
from bigml.predict_utils.common import OPERATION_OFFSET, FIELD_OFFSET, \
    VALUE_OFFSET, MISSING_OFFSET
from bigml.generators.tree_common import filter_nodes


T_MISSING_OPERATOR = {
    "=": "ISNULL(",
    "!=": "NOT ISNULL("
}


def value_to_print(value, optype):
    """String of code that represents a value according to its type

    """
    if value is None:
        return "NULL"
    if optype == 'numeric':
        return value
    return "'%s'" % value.replace("'", '\\\'')


def missing_check_code(tree, offsets, fields, objective_id,
                       field, alternate, cmv, conditions, attr=None):
    """Builds the code to predict when the field is missing

    """
    node = get_node(tree)
    conditions.append("ISNULL([%s])" % fields[field]['name'])
    code = "%s %s THEN " % \
        (alternate, " AND ".join(conditions))
    if attr is None:
        value = value_to_print( \
            node[offsets["output"]], fields[objective_id]['optype'])
    else:
        value = node[offsets[attr]]
    code += ("%s\n" % value)
    cmv.append(fields[field]['name'])
    del conditions[-1]

    return code

def missing_prefix_code(tree, fields, field, cmv):
    """Part of the condition that checks for missings when missing_splits
    has been used

    """

    predicate = get_predicate(tree)
    missing = predicate[MISSING_OFFSET]
    negation = "" if missing else "NOT "
    connection = "OR" if missing else "AND"
    if not missing:
        cmv.append(fields[field]['name'])
    return "(%sISNULL([%s]) %s " % ( \
        negation, fields[field]['name'],
        connection)

def split_condition_code(tree, fields, field, conditions,
                         pre_condition, post_condition):
    """Condition code for the split

    """
    predicate = get_predicate(tree)
    optype = fields[field]['optype']
    value = value_to_print(predicate[VALUE_OFFSET], optype)
    operator = ("" if predicate[VALUE_OFFSET] is None else
                PYTHON_OPERATOR[predicate[OPERATION_OFFSET]])
    if predicate[VALUE_OFFSET]  is None:
        pre_condition = (
            T_MISSING_OPERATOR[predicate[OPERATION_OFFSET]])
        post_condition = ")"

    conditions.append("%s[%s]%s%s%s" % (
        pre_condition,
        fields[predicate[FIELD_OFFSET]]['name'],
        operator,
        value,
        post_condition))

def plug_in_body(tree, offsets, fields, objective_id,
                 body="", conditions=None, cmv=None,
                 ids_path=None, subtree=True, attr=None):
    """Translate the model into a set of "if" statemets in tableau syntax

    `depth` controls the size of indentation. As soon as a value is missing
    that node is returned without further evaluation.

    """

    if cmv is None:
        cmv = []

    if body:
        alternate = "ELSEIF"
    else:
        if conditions is None:
            conditions = []
        alternate = "IF"

    node = get_node(tree)

    children = filter_nodes([] if node[offsets["children#"]] == 0 \
        else node[offsets["children"]], offsets, ids=ids_path, subtree=subtree)

    if children:

        field = mintree_split(children)
        has_missing_branch = (missing_branch(children) or
                              none_value(children))
        # the missing is singled out as a special case only when there's
        # no missing branch in the children list
        one_branch = not has_missing_branch or \
            fields[field]['optype'] in COMPOSED_FIELDS
        if (one_branch and
                not fields[field]['name'] in cmv):
            body += missing_check_code(tree, offsets, fields, objective_id,
                                       field, alternate, cmv,
                                       conditions, attr=attr)
            alternate = "ELSEIF"

        for child in children:
            pre_condition = ""
            post_condition = ""
            predicate = get_predicate(child)
            [_, field, value, _, _] = predicate
            if has_missing_branch and value is not None:
                pre_condition = missing_prefix_code(child, fields, field, cmv)
                post_condition = ")"

            split_condition_code(child, fields, field, conditions,
                                 pre_condition, post_condition)

            body = plug_in_body(child, offsets, fields, objective_id,
                                body, conditions[:], cmv=cmv[:],
                                ids_path=ids_path, subtree=subtree,
                                attr=attr)
            del conditions[-1]
    else:
        if attr is None:
            value = value_to_print( \
                node[offsets["output"]], fields[objective_id]['optype'])
        else:
            value = node[offsets[attr]]
        body += "%s %s THEN" % (alternate, " AND ".join(conditions))
        body += " %s\n" % value

    return body



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2017-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Bigmler retrain

"""
import os

from bigml.api import BigML

from bigmler.tests.world import (world, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.project_creation_steps as test_project


try:
    BIGML_ORGANIZATION = os.environ['BIGML_ORGANIZATION']
except KeyError:
    raise ValueError("You need to set BIGML_ORGANIZATION"
                     " to an organization ID in your "
                     "environment variables to run this test.")


def setup_module():
    """Setup for the module

    """
    world.bck_api = world.api
    world.api = BigML(world.username, world.api_key, debug=world.debug,
                      organization=BIGML_ORGANIZATION)
    print(world.api.connection_info())
    world.bck_project_id = world.project_id
    world.project_id = None
    world.clear()


def teardown_module():
    """Teardown for the module

    """

    if not world.debug:
        world.api = BigML(world.username, world.api_key, debug=world.debug,
                          organization=BIGML_ORGANIZATION)
        world.project_id = world.project["resource"]
        world.api.delete_project(world.project_id)
    world.project_id = world.bck_project_id
    world.api = world.bck_api
    print(world.api.connection_info())


class TestProjectOrg:
    """Testing Projects in Organizations"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully creating a project and associating resources to it:
            Given I create a BigML project in an organization named "<project>" storing results in "<output_dir>"
            And I create a BigML source uploading train "<data>" file and associate it to the organization project storing results in "<output_dir>"
            And I check that the project has been created
            And I check that the source has been created
            Then the source is associated to the project

            Examples:
            | data             | project         | output_dir
            | ../data/iris.csv | My new project  | ./scenario_org_1
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "project", "output_dir"]
        examples = [
            ['data/iris.csv', 'My new project', 'scenario_org_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_project.i_create_project_in_org(
                self, name=example["project"],
                output_dir=example["output_dir"],
                organization=BIGML_ORGANIZATION)
            test_project.i_check_create_project(self, organization=True)
            test_project.i_create_source_with_org_project(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_project.check_source_in_project(self)



# -*- coding: utf-8 -*-
#
# Copyright 2017-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - Resources processing: creation, update and retrieval
of deepnets

"""


from bigml.fields import Fields, DEFAULT_MISSING_TOKENS

import bigmler.utils as u
import bigmler.resourcesapi.deepnets as r
import bigmler.checkpoint as c


def has_deepnet(args):
    """Returns whether some kind of deepnet

    """
    return args.deepnet or args.deepnets or \
        args.deepnet_tag


def deepnets_processing(datasets, deepnets, \
    deepnet_ids, api, args, resume, fields=None, \
    session_file=None, path=None, log=None):
    """Creates or retrieves deepnet from the input data

    """

    # If we have a dataset but not a model, we create the model if the no_model
    # flag hasn't been set up.
    if datasets and not (has_deepnet(args) or \
            args.no_deepnet):
        deepnet_ids = []
        deepnets = []

        # Only 1 deepnet per bigmler command at present
        number_of_deepnets = 1
        if resume:
            resume, deepnet_ids = c.checkpoint( \
                c.are_deepnets_created, path, \
                number_of_deepnets, debug=args.debug)
            if not resume:
                message = u.dated("Found %s deepnets out of %s."
                                  " Resuming.\n"
                                  % (len(deepnet_ids),
                                     number_of_deepnets))
                u.log_message(message, log_file=session_file,
                              console=args.verbosity)

            deepnets = deepnet_ids
            number_of_deepnets -= len(deepnet_ids)

        deepnet_args = r.set_deepnet_args( \
            args, fields=fields, \
            deepnet_fields=args.deepnet_fields_,
            objective_id=args.objective_id_)
        deepnets, deepnet_ids = \
            r.create_deepnets( \
            datasets, deepnets, deepnet_args, \
            args, api, path, session_file, log)
    # If a deepnet is provided, we use it.
    elif args.deepnet:
        deepnet_ids = [args.deepnet]
        deepnets = deepnet_ids[:]

    elif args.deepnets or args.deepnet_tag:
        deepnets = deepnet_ids[:]

    # If we are going to predict we must retrieve the deepnets
    if deepnet_ids and (args.test_set or args.export_fields):
        deepnets, deepnet_ids = \
            r.get_deepnets(deepnets, args, api, \
            session_file)

    return deepnets, deepnet_ids, resume


def get_deepnet_fields(deepnet, csv_properties, args):
    """Retrieves fields info from deepnet resource

    """
    if not csv_properties:
        csv_properties = {}
    csv_properties.update(verbose=True)
    if args.user_locale is None:
        args.user_locale = deepnet['object'].get('locale', None)
    csv_properties.update(data_locale=args.user_locale)
    csv_properties.update(missing_tokens=DEFAULT_MISSING_TOKENS)
    csv_properties.update(objective_field=deepnet['object'].get( \
        'objective_field'))
    return Fields(deepnet['object'][ \
        'deepnet']['fields'], **csv_properties)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_batch_args, map_fields, \
    update_json_args

from bigmler.resourcesapi.common import FULL_FORMAT


def set_batch_topic_distribution_args( \
    args, fields=None, dataset_fields=None):
    """Return batch topic distribution args dict

    """
    batch_topic_distribution_args = set_basic_batch_args(args, args.name)

    if args.fields_map_ and fields is not None:
        if dataset_fields is None:
            dataset_fields = fields
        batch_topic_distribution_args.update({
            "fields_map": map_fields(args.fields_map_,
                                     fields, dataset_fields)})

    if args.prediction_info == FULL_FORMAT:
        batch_topic_distribution_args.update(all_fields=True)
    if args.prediction_fields:
        batch_topic_distribution_args.update(all_fields=False)
        prediction_fields = []
        for field in args.prediction_fields.split(args.args_separator):
            field = field.strip()
            if not field in dataset_fields.fields:
                try:
                    field = dataset_fields.field_id(field)
                except Exception as exc:
                    sys.exit(exc)
            prediction_fields.append(field)
        batch_topic_distribution_args.update(output_fields=prediction_fields)
    if 'batch_topic_distribution' in args.json_args:
        update_json_args(
            batch_topic_distribution_args, args.json_args.get( \
                'batch_topic_distribution'), fields)

    return batch_topic_distribution_args


def create_batch_topic_distribution(topic_model, test_dataset,
                                    batch_topic_distribution_args, args,
                                    api=None, session_file=None,
                                    path=None, log=None):
    """Creates remote batch topic distribution

    """
    if api is None:
        api = bigml.api.BigML()
    message = dated("Creating batch topic distribution.\n")
    log_message(message, log_file=session_file, console=args.verbosity)
    batch_topic_distribution = api.create_batch_topic_distribution( \
        topic_model, test_dataset, batch_topic_distribution_args, retries=None)
    log_created_resources( \
        "batch_topic_distribution", path,
        bigml.api.get_batch_topic_distribution_id(batch_topic_distribution),
        mode='a')
    batch_topic_distribution_id = check_resource_error(
        batch_topic_distribution,
        "Failed to create batch topic distribution: ")
    try:
        batch_topic_distribution = check_resource( \
            batch_topic_distribution, api.get_batch_topic_distribution,
            raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished batch topic distribution: %s"
                 % str(exception))
    message = dated("Batch topic distribution created: %s\n"
                    % get_url(batch_topic_distribution))
    log_message(message, log_file=session_file, console=args.verbosity)
    log_message("%s\n" % batch_topic_distribution_id, log_file=log)
    if args.reports:
        report(args.reports, path, batch_topic_distribution)
    return batch_topic_distribution



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2014-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os
import csv

from subprocess import check_call, CalledProcessError

from bigml.api import HTTP_OK

from bigmler.checkpoint import file_number_of_lines
from bigmler.tests.world import world, approx_error, ok_, ok_error


def check_debug(command, project=True):
    """Adds verbosity level and command print.

    """
    non_project_commands = ["--project", "bigmler reify", "bigmler analyze",
                            "bigmler delete", "bigmler report",
                            "bigmler delete", "bigmler project",
                            "bigmler execute", "bigmler whizzml",
                            "bigmler export", "bigmler retrain",
                            "--org-project"]
    debug = os.environ.get('BIGMLER_DEBUG', False)

    if command.find("bigmler") < 0:
        if debug:
            print(command)
        return command

    # adding project id as source creation parameter
    if (project and
        all(command.find(string) < 0 for string in non_project_commands)):
        command = "%s --project-id %s" % (command, world.project_id)
    verbosity = 0
    extend_cmd = ''
    if debug == '1':
        verbosity = 1
    elif debug == '2':
        extend_cmd = ' --debug'
    if command.find("bigmler report") < 0:
        command = "%s --verbosity %s%s" % (command, verbosity, extend_cmd)
    if debug:
        print(command)
    return command


def check_http_code(resources):
    """Checks the http code in the resource list

    """
    if resources['code'] == HTTP_OK:
        assert True
    else:
        assert False, "Response code: %s" % resources['code']


def store_init_resources():
    """Store the initial existing resources grouped by resource_type

    """
    world.count_resources('init')


def store_final_resources():
    """Store the final existing resources grouped by resource_type

    """
    world.count_resources('final')


def check_init_equals_final():
    """Checks that the number of resources grouped by type has not changed

    """
    world.check_init_equals_final()


def shell_execute(command, output, test=None, options=None,
                  data=None, test_split=None, test_rows=None, project=True):
    """Excute bigmler command in shell and checking test rows"""
    world.output = output
    message = base_shell_execute(
        command, os.path.dirname(output), project=project)
    try:
        world.test_lines = 0
        if test is not None:
            world.test_header = 1
            world.test_lines += file_number_of_lines(test)
            # if test file containss headers and the prediction command don't,
            # we need to remove 1 to compensate for the header
            if '_nh' not in test and '--prediction-header' not in command \
                    and '--remote' not in command:
                world.test_lines -= 1
                world.test_header = 0
            if '_nh' not in test and options is not None and \
                    '--no-header' in options:
                world.test_lines -= 1
                world.test_header = 0
        elif test_rows is not None:
            world.test_header = 0
            world.test_lines += test_rows
        elif test_split is not None:
            world.test_header = 0
            data_lines = file_number_of_lines(data) - 1
            world.test_lines = int(data_lines * float(test_split))
    except IOError as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def base_shell_execute(command, output_dir, project=True):
    """Executing bigmler command in shell"""
    message = None
    command = check_debug(command, project=project)
    world.directory = output_dir
    world.folders.append(output_dir)
    try:
        retcode = check_call(command, shell=True)
        ok_(retcode >= 0)
    except (OSError, CalledProcessError, IOError) as exc:
        message = str(exc)
    return message


def check_rows_equal(reader_file, check_file):
    """Checking values in rows are equal"""
    message = None
    with open(reader_file) as p_handler:
        reader = csv.reader(p_handler, lineterminator="\n")
        with open(check_file) as c_handler:
            check_reader = csv.reader(c_handler, lineterminator="\n")
            for index0, row in enumerate(reader):
                ref_row = next(check_reader)
                if len(ref_row) != len(row):
                    return "Row %s with different lengths: %s, %s, %s, %s" % (
                        str(index0), str(len(ref_row)), str(len(row)), ref_row,
                        row)
                for index, row_value in enumerate(row):
                    dot = row_value.find(".")
                    if dot > 0 or (ref_row[index].find(".") > 0
                                   and ref_row[index].endswith(".0")):
                        try:
                            decimal_places = min((
                                min(len(row_value), len(ref_row[index]))
                                - dot - 1), 5)
                            message = approx_error(
                                float(row_value),
                                float(ref_row[index]),
                                precision=decimal_places,
                                msg=("Row %s don't match. Found %s,"
                                     " %s expected" % (
                                     index0, row_value, ref_row[index])))
                        except ValueError:
                            message = ok_error(row_value == ref_row[index],
                                msg=("Row %s don't match. Found %s,"
                                     " %s expected" % (
                                     index0, row_value, ref_row[index])))
                    else:
                        message = ok_error(row_value == ref_row[index],
                            msg=("Row %s don't match. Found %s,"
                                 " %s expected" % (
                                 index0, row_value, ref_row[index])))
                    if message is not None:
                        break
                if message is not None:
                    break
    return message



# -*- coding: utf-8 -*-
#
# Copyright 2019-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


"""Options for BigMLer Fusion

"""

def get_fusion_options(defaults=None):
    """Adding arguments for the fusion subcommand

    """

    if defaults is None:
        defaults = {}

    options = {
        # If a list of BigML models is provided, the script will
        # create a Fusion from them
        '--fusion-models': {
            "action": 'store',
            "dest": 'fusion_models',
            "default": defaults.get('fusion_models', None),
            "help": ("Comma-separated list of models to be included "
                     "in the Fusion resource.")},

        # If a path to the JSON of model maps, the script will
        # create a Fusion from it
        '--fusion-models-file': {
            "action": 'store',
            "dest": 'fusion_models_file',
            "default": defaults.get('fusion_models_file', None),
            "help": ("Path to a JSON file that contains a list "
                     "of model IDs or maps to include in the "
                     "Fusion resource.")},

        # If a BigML Fusion is provided, the script will
        # use it to generate predictions
        '--fusion': {
            'action': 'store',
            'dest': 'fusion',
            'default': defaults.get('fusion', None),
            'help': "BigML Fusion Id."},

        # The path to a file containing fusion ids.
        '--fusions': {
            'action': 'store',
            'dest': 'fusions',
            'default': defaults.get('fusions', None),
            'help': ("Path to a file containing fusion/ids."
                     " One Fusion"
                     " per line (e.g., "
                     "fusion/50a206a8035d0706dc000376"
                     ").")},

        # If a BigML JSON file containing a Fusion
        # structure is provided,
        # the script will use it.
        '--fusion-file': {
            'action': 'store',
            'dest': 'fusion_file',
            'default': defaults.get('fusion_file', None),
            'help': "BigML Fusion JSON structure file."},

        # The path to a file containing Fusion attributes.
        '--fusion-attributes': {
            'action': 'store',
            'dest': 'fusion_attributes',
            'default': defaults.get('fusion_attributes', None),
            'help': ("Path to a json file describing Fusion"
                     " attributes.")}}

    return options



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
# Copyright 2017-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Tree level output for Tableau

This module defines functions that generate Tableau code to make local
predictions
"""
import sys

from bigml.model import Model
from bigml.generators.model import get_ids_path

from bigmler.export.out_tree.tableautree import plug_in_body


class TableauModel(Model):
    """Tableau Model output generator class """

    def __init__(self, model, api=None, fields=None):
        """Empty attributes to be overriden

        """
        Model.__init__(self, model, api, fields)

    def plug_in(self, out=sys.stdout,
                filter_id=None, subtree=True, attr=None):
        """Returns a basic Tableau function that implements the model.

        `out` is file descriptor to write the tableau code.

        """
        ids_path = get_ids_path(self, filter_id)
        length = self.tableau(out, ids_path=ids_path,
                              subtree=subtree, attr=attr)
        if length > 0:
            out.write("END\n")
        else:
            sys.exit("\nFailed to represent this model in "
                     "in Tableau syntax. Currently, only "
                     "models with categorical and numeric fields "
                     "can be generated.\n")
        out.flush()

    def tableau(self, out, ids_path=None, subtree=True, attr=None):
        """Writes a Tableau function that implements the model.

        """
        body = plug_in_body(self.tree, self.offsets, self.fields,
                            self.objective_id, ids_path=ids_path,
                            subtree=subtree, attr=attr)
        out.write(body)
        out.flush()
        return len(body)



# -*- coding: utf-8 -*-
#pylint: disable=locally-disabled,unused-argument,no-member
#
# Copyright 2019-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import os


from bigml.api import check_resource

import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.basic_deepnet_steps as test_dn

from bigmler.tests.common_steps import shell_execute
from bigmler.tests.world import world, res_filename, ok_


def i_create_fs_resources_from_model(step, test=None, output=None):
    """Step: I create BigML resources using model to test <test> and log
    predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" +
               test + "\" --store --output " +
               output)
    shell_execute(command, output, test=test)


def i_create_fs_resources_from_model_remote(step, test=None, output=None):
    """Step: I create BigML resources using model to test <test> as batch
    prediction and log predictions in <output>
    """
    ok_(test is not None and output is not None)
    test = res_filename(test)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" + test +
               "\" --store --remote --output " +
               output)
    shell_execute(command, output, test=test, options="--no-header")


def i_create_fs_resources_from_model_remote_with_options(
    step, test=None, output=None, options_file=None):
    """Step: I create BigML resources using model to test <test> as batch
    prediction with output format <options_file> and log predictions in
    <output>
    """
    ok_(test is not None and output is not None and options_file is not None)
    test = res_filename(test)
    options_file = res_filename(options_file)

    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test \"" + test +
               "\" --batch-prediction-attributes " + options_file +
               " --store --remote --output " +
               output)
    shell_execute(command, output, test=test, options="--no-header")


def i_create_fs_resources_from_mode_and_evaluate(step, output=None):
    """Step: I create BigML resources using model to test <test> and an
    evaluation and log predictions in <output>
    """
    ok_(output is not None)
    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " +
               ",".join(models) + " --test-dataset \"" +
               world.dataset["resource"] +
               "\" --evaluate --store --remote --output " +
               output)
    shell_execute(command, output, options="--no-header")


def i_check_create_fusion(step):
    """Step: I check that the fusion model has been created"""
    fs_file = os.path.join(world.directory, "fusions")
    message = None
    try:
        with open(fs_file) as handler:
            fusion = check_resource(handler.readline().strip(),
                                    world.api.get_fusion)
            world.fusions.append(fusion['resource'])
            world.fusion = fusion
    except Exception as exc:
        message = str(exc)
    ok_(message is None, msg=message)


def setup_for_fusion(step, train=None, output_dir=None):
    """Creating models needed for a fusion"""
    train = res_filename(train)
    command = ("bigmler --train \"" +
               train + "\" --store --output-dir " +
               output_dir)
    shell_execute(command, "%s/predictions" % output_dir)
    test_pred.i_check_create_source(step)
    test_pred.i_check_create_dataset(step)
    test_pred.i_check_create_model(step)

    command = ("bigmler deepnet --dataset \"" +
               world.dataset["resource"] + "\" --store --output-dir " +
               output_dir)
    shell_execute(command, "%s/predictions" % output_dir)
    test_dn.i_check_create_dn_model(step)


def i_create_all_fs_resources_to_evaluate(step, train=None, test=None, output=None):
    """Step: I create BigML fusion resources uploading train <train> file to
    evaluate and log evaluation in <output>
    """
    models = [world.model["resource"], world.deepnet["resource"]]
    command = ("bigmler fusion --fusion-models " + ",".join(models) +
               " --test \"" +
               test +
               "\" --evaluate" +
               " --store --output " + output)
    shell_execute(command, output)



# -*- coding: utf-8 -*-
#
# Copyright 2020-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""


import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           plural, is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_args, \
    configure_input_fields, update_sample_parameters_args, \
    update_json_args, wait_for_available_tasks

from bigmler.resourcesapi.common import SEED, FIELDS_QS, \
    ALL_FIELDS_QS


def set_pca_args(args, name=None, fields=None,
                 pca_fields=None):
    """Return pca arguments dict

    """
    if name is None:
        name = args.name
    if pca_fields is None:
        pca_fields = args.pca_fields_

    pca_args = set_basic_args(args, name)
    pca_args.update({
        "seed": SEED if args.seed is None else args.seed,
        "pca_seed": SEED if args.seed is None else args.seed
    })

    pca_args.update({"sample_rate": args.sample_rate})
    pca_args = update_sample_parameters_args( \
        pca_args, args)
    if fields is not None:
        input_fields = list(fields.fields.keys())
    if pca_fields and fields is not None:
        input_fields = configure_input_fields(fields, pca_fields)
    if args.exclude_objective:
        input_fields = [field for field in input_fields \
            if field not in args.exclude_fields]
    pca_args.update(input_fields=input_fields)

    if 'pca' in args.json_args:
        update_json_args(pca_args,
                         args.json_args.get('pca'),
                         fields)
    return pca_args


def create_pca(datasets, pca, pca_args,
               args, api=None, path=None,
               session_file=None, log=None):
    """Create remote pcas

    """
    if api is None:
        api = bigml.api.BigML()

    pcas = []
    pca_ids = []
    if pca is not None:
        pcas = [pca]
        pca_ids = [pca]
    existing_pcas = len(pcas)
    pca_args_list = []
    datasets = datasets[existing_pcas:]
    # if resuming and all pcas were created, there will
    # be no datasets left
    if datasets:
        if isinstance(pca_args, list):
            pca_args_list = pca_args

        # Only one pca per command, at present
        number_of_pcas = 1
        message = dated("Creating %s.\n" %
                        plural("pca", number_of_pcas))
        log_message(message, log_file=session_file,
                    console=args.verbosity)

        query_string = FIELDS_QS
        inprogress = []
        for i in range(0, number_of_pcas):
            wait_for_available_tasks(inprogress,
                                     args.max_parallel_pcas,
                                     api, "pca")
            if pca_args_list:
                pca_args = pca_args_list[i]

            pca = api.create_pca(datasets,
                                 pca_args,
                                 retries=None)
            pca_id = check_resource_error( \
                pca,
                "Failed to create pca: ")
            log_message("%s\n" % pca_id, log_file=log)
            pca_ids.append(pca_id)
            inprogress.append(pca_id)
            pcas.append(pca)
            log_created_resources("pcas", path, pca_id,
                                  mode='a')

        if args.verbosity:
            if bigml.api.get_status(pca)['code'] != bigml.api.FINISHED:
                try:
                    pca = check_resource( \
                        pca, api.get_pca,
                        query_string=query_string,
                        raise_on_error=True)
                except Exception as exception:
                    sys.exit("Failed to get a finished pca: %s" %
                             str(exception))
                pcas[0] = pca
            message = dated("PCA created: %s\n" %
                            get_url(pca))
            log_message(message, log_file=session_file,
                        console=args.verbosity)
            if args.reports:
                report(args.reports, path, pca)

    return pca


def get_pca(pca,
            args, api=None, session_file=None):
    """Retrieves remote pca in its actual status

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Retrieving PCA. %s\n" %
                    get_url(pca))
    log_message(message, log_file=session_file, console=args.verbosity)
    # only one PCA at present
    try:
        # we need the whole fields structure when exporting fields
        query_string = FIELDS_QS if not args.export_fields else ALL_FIELDS_QS
        pca = check_resource(pca,
                             api.get_pca,
                             query_string=query_string,
                             raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished pca: %s" % \
            str(exception))

    return pca


def set_publish_pca_args(args):
    """Set args to publish pca

    """
    public_pca = {}
    if args.public_pca:
        public_pca = {"private": False}
        if args.model_price:
            public_pca.update(price=args.model_price)
        if args.cpp:
            public_pca.update(credits_per_prediction=args.cpp)
    return public_pca


def update_pca(pca, pca_args,
               args, api=None, path=None, session_file=None):
    """Updates pca properties

    """
    if api is None:
        api = bigml.api.BigML()

    message = dated("Updating PCA. %s\n" %
                    get_url(pca))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    pca = api.update_pca(pca, pca_args)
    check_resource_error(pca,
                         "Failed to update PCA: %s"
                         % pca['resource'])
    pca = check_resource(pca,
                         api.get_pca,
                         query_string=FIELDS_QS,
                         raise_on_error=True)
    if is_shared(pca):
        message = dated("Shared PCA link. %s\n" %
                        get_url(pca, shared=True))
        log_message(message, log_file=session_file, console=args.verbosity)
        if args.reports:
            report(args.reports, path, pca)

    return pca



# -*- coding: utf-8 -*-
#
# Copyright 2020-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Resources management functions

"""



import sys

import bigml.api

from bigmler.utils import (dated, get_url, log_message, check_resource,
                           is_shared,
                           check_resource_error, log_created_resources)
from bigmler.reports import report

from bigmler.resourcesapi.common import set_basic_args, update_attributes, \
    update_json_args, configure_input_fields, \
    check_fields_struct

from bigmler.resourcesapi.common import SEED, DS_NAMES, \
    ALL_FIELDS_QS


def set_basic_dataset_args(args, name=None):
    """Return dataset basic arguments dict

    """
    if name is None and args.name is not None:
        name = args.name
    dataset_args = set_basic_args(args, name)
    if args.sample_rate != 1 and args.no_model:
        dataset_args.update({
            "seed": SEED if args.seed is None else args.seed,
            "sample_rate": args.sample_rate
        })

    if hasattr(args, "range") and args.range_:
        dataset_args.update({
            "range": args.range_
        })
    return dataset_args


def set_dataset_args(args, fields, multi_label_data=None):
    """Return dataset arguments dict

    """
    dataset_args = set_basic_dataset_args(args)
    objective_field = (None if not hasattr(args, 'objective_field')
                       else args.objective_field)
    if multi_label_data is not None and objective_field is None:
        objective_field = multi_label_data['objective_name']
    if objective_field is not None and fields is not None:
        try:
            objective_id = fields.field_id(objective_field)
        except ValueError as exc:
            sys.exit(exc)
        dataset_args.update(objective_field={'id': objective_id})

    if hasattr(args, 'juxtapose') and args.juxtapose:
        dataset_args.update({"juxtapose": args.juxtapose})

    if hasattr(args, 'sql_query') and args.sql_query:
        dataset_args.update({"sql_query": args.sql_query})

    if hasattr(args, 'sql_output_fields_') and args.sql_output_fields_:
        dataset_args.update({"sql_output_fields": args.sql_output_fields_})

    if hasattr(args, 'json_query_') and args.json_query_:
        dataset_args.update({"json_query": args.json_query_})

    if args.json_filter:
        dataset_args.update(json_filter=args.json_filter)
    elif args.lisp_filter:
        dataset_args.update(lisp_filter=args.lisp_filter)

    if args.dataset_fields_ and fields is not None:
        input_fields = configure_input_fields(fields, args.dataset_fields_)
        dataset_args.update(input_fields=input_fields)
    if (hasattr(args, 'multi_label') and args.multi_label
            and multi_label_data is not None):
        dataset_args.update(
            user_metadata={'multi_label_data': multi_label_data})

    if fields and args.import_fields:
        fields_struct = fields.new_fields_structure(args.import_fields)
        check_fields_struct(fields_struct, "dataset")
        update_attributes(dataset_args, fields_struct)
    if 'dataset' in args.json_args:
        update_json_args(dataset_args, args.json_args.get('dataset'), fields)

    return dataset_args


def set_dataset_split_args(name, description, args, sample_rate=1,
                           out_of_bag=False, multi_label_data=None):
    """Return dataset arguments dict to split a dataset

    """
    dataset_args = {
        "name": name,
        "description": description,
        "category": args.category,
        "tags": args.tag,
        "seed": SEED if args.seed is None else args.seed,
        "sample_rate": sample_rate,
        "out_of_bag": out_of_bag
    }

    if hasattr(args, "range") and args.range_:
        dataset_args.update({
            "range": args.range_
        })
    if (hasattr(args, "multi_label") and
            args.multi_label and multi_label_data is not None):
        dataset_args.update(
            user_metadata={'multi_label_data': multi_label_data})

    return dataset_args


def create_dataset(origin_resource, dataset_args, args, api=None,
                   path=None, session_file=None, log=None, dataset_type=None):
    """Creates remote dataset from source, dataset, cluster or datasets list

    """
    if api is None:
        api = bigml.api.BigML()
    message = dated("Creating dataset.\n")
    log_message(message, log_file=session_file, console=args.verbosity)
    check_fields_struct(dataset_args, "dataset")

    # if --json-query or --sql-query are used and no names are set for
    # the datasets, we create default naming to A, B, C, etc. for the datasets
    # to be used as origin

    #pylint: disable=locally-disabled,too-many-boolean-expressions
    if ((hasattr(args, 'sql_query') and args.sql_query) or \
            (hasattr(args, 'json_query') and args.sql_query)) and \
            isinstance(origin_resource, list) and \
            ((not isinstance(origin_resource[0], dict)) or \
            origin_resource[0].get("name") is None):
        for index, element in enumerate(origin_resource):
            if index < len(DS_NAMES):
                if isinstance(element, dict):
                    if element.get("resource") is not None:
                        element = {"id": element["resource"]}
                    element.update({"name": DS_NAMES[index]})
                    origin_resource[index] = element
                elif isinstance(element, str):
                    origin_resource[index] = {"id": element,
                                              "name": DS_NAMES[index]}

    dataset = api.create_dataset(origin_resource, dataset_args, retries=None)
    suffix = "_" + dataset_type if dataset_type else ""
    log_created_resources("dataset%s" % suffix, path,
                          bigml.api.get_dataset_id(dataset), mode='a')
    dataset_id = check_resource_error(dataset, "Failed to create dataset: ")
    try:
        dataset = check_resource(dataset, api.get_dataset,
                                 query_string=ALL_FIELDS_QS,
                                 raise_on_error=True)
    except Exception as exception:
        sys.exit("Failed to get a finished dataset: %s" % str(exception))
    message = dated("Dataset created: %s\n" % get_url(dataset))
    log_message(message, log_file=session_file, console=args.verbosity)
    log_message("%s\n" % dataset_id, log_file=log)
    if args.reports:
        report(args.reports, path, dataset)
    return dataset


def get_dataset(dataset, api=None, verbosity=True, session_file=None):
    """Retrieves the dataset in its actual state

    """
    if api is None:
        api = bigml.api.BigML()
    if (isinstance(dataset, str) or
            bigml.api.get_status(dataset)['code'] != bigml.api.FINISHED):
        message = dated("Retrieving dataset. %s\n" %
                        get_url(dataset))
        log_message(message, log_file=session_file,
                    console=verbosity)
        dataset = check_resource(dataset, api.get_dataset,
                                 query_string=ALL_FIELDS_QS)
        check_resource_error(dataset, "Failed to get dataset: ")
    return dataset


def publish_dataset(dataset, args, api=None, session_file=None):
    """Publishes dataset and sets its price (if any)

    """
    if api is None:
        api = bigml.api.BigML()
    public_dataset = {"private": False}
    if args.dataset_price:
        public_dataset.update(price=args.dataset_price)
    dataset = update_dataset(dataset, public_dataset, args, api=api,
                             session_file=session_file)
    check_resource_error(dataset, "Failed to update dataset: ")
    dataset = check_resource(dataset, api.get_dataset,
                             query_string=ALL_FIELDS_QS)
    return dataset

def update_dataset(dataset, dataset_args, args,
                   api=None, path=None, session_file=None):
    """Updates dataset properties

    """
    if api is None:
        api = bigml.api.BigML()
    message = dated("Updating dataset. %s\n" %
                    get_url(dataset))
    log_message(message, log_file=session_file,
                console=args.verbosity)
    dataset = api.update_dataset(dataset, dataset_args)
    if is_shared(dataset):
        message = dated("Shared dataset link. %s\n" %
                        get_url(dataset, shared=True))
        log_message(message, log_file=session_file,
                    console=args.verbosity)
        if args.reports:
            report(args.reports, path, dataset)
    check_resource_error(dataset, "Failed to update dataset: ")
    dataset = check_resource(dataset, api.get_dataset,
                             query_string=ALL_FIELDS_QS)

    return dataset



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#
# Copyright 2015-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing delete subcommand

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)


import bigmler.tests.basic_tst_prediction_steps as test_pred
import bigmler.tests.delete_subcommand_steps as test_delete


def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestDelete:
    """Testing delete command """

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully deleting a source by id:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source exists
            And I delete the source by id using --ids storing results in "<output_dir>"
            Then I check that the source doesn't exist
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_del_1']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)
            test_delete.i_delete_source_by_ids(
                self, output_dir=example["output_dir"])
            test_delete.i_check_source_does_not_exist(
                self, source_id=None)

    def test_scenario2(self):
        """
        Scenario: Failing deleting a source by id when --dry-run is used:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I delete the source by id using --ids and --dry-run storing results in "<output_dir>"
            Then I check that the source exists
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_del_2']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_delete.i_delete_source_by_ids_dry(
                self, output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)

    def test_scenario3(self):
        """
        Scenario: Failing deleting a source by id when a different resource_types is used:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I delete the source by id using --ids and --resource-types "<resource_types>" storing results in "<output_dir>"
            Then I check that the source exists
        """
        print(self.test_scenario3.__doc__)
        headers = ["data", "output_dir", "resource_types"]
        examples = [
            ['data/iris.csv', 'scenario_del_3', 'dataset,model']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_delete.i_delete_source_by_ids_filtered(
                self, resource_types=example["resource_types"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)

    def test_scenario4(self):
        """
        Scenario: Successfully deleting a source from a file:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source exists
            And I delete the source by id using --from-file and the source file storing results in "<output_dir>"
            Then I check that the source doesn't exist
        """
        print(self.test_scenario4.__doc__)
        headers = ["data", "output_dir"]
        examples = [
            ['data/iris.csv', 'scenario_del_4']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)
            test_delete.i_delete_source_by_file(
                self, output_dir=example["output_dir"])
            test_delete.i_check_source_does_not_exist(self, source_id=None)

    def test_scenario5(self):
        """
            Scenario: Failing deleting a source from a file when a different resource_types is used:
                Given I create a BigML source from file "<data>" storing results in "<output_dir>"
                And I check that the source has been created
                And I delete the source by id using --from-file, the source file and --resource-types "<resource_types>" storing results in "<output_dir>"
                Then I check that the source exists
        """
        print(self.test_scenario5.__doc__)
        headers = ["data", "output_dir", "resource_types"]
        examples = [
            ['data/iris.csv', 'scenario_del_5', 'dataset,model']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            test_delete.i_delete_source_by_file_filtered(
                self, resource_types=example["resource_types"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)

    def test_scenario6(self):
        """
            Scenario: Sucessfully deleting a source in a time range:
                Given I create a BigML source from file "<data>" storing results in "<output_dir>"
                And I check that the source has been created
                And I store the source id as lower
                And I create a BigML source from file "<data>" storing results in "<output_dir2>"
                And I check that the source exists
                And I store the source id as reference
                And I create a BigML source from file "<data>" storing results in "<output_dir3>"
                And I check that the source has been created
                And I store the source id as upper
                And I delete the source using --older-than and --newer-than storing results in "<output_dir>"
                Then I check that the reference source doesn't exist
        """
        print(self.test_scenario6.__doc__)
        headers = ["data", "output_dir", "output_dir2", "output_dir3"]
        examples = [
            ['data/iris.csv', 'scenario_del_6', 'scenario_del_6_2',
             'scenario_del_6_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_lower"] = world.source["resource"]
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir2"])
            test_delete.i_check_source_exists(self)
            self.bigml["source_reference"] = world.source["resource"]
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir3"])
            test_pred.i_check_create_source(self)
            self.bigml["source_upper"] = world.source["resource"]
            test_delete.i_delete_source_older_newer(
                self, output_dir=example["output_dir3"])
            test_delete.i_check_source_does_not_exist(
                self, source_id=self.bigml["source_reference"])

    def test_scenario7(self):
        """
        Scenario: Failing deleting a source in a time range when a different resource_types is used:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I store the source id as lower
            And I create a BigML source from file "<data>" storing results in "<output_dir2>"
            And I check that the source has been created
            And I store the source id as reference
            And I create a BigML source from file "<data>" storing results in "<output_dir3>"
            And I check that the source has been created
            And I store the source id as upper
            And I delete the source using --older-than and --newer-than with resource_types "<resource_types>" storing results in "<output_dir>"
            Then I check that the reference source exists
        """
        print(self.test_scenario7.__doc__)
        headers = ["data", "output_dir", "resource_types", "output_dir2",
                   "output_dir3"]
        examples = [
            ['data/iris.csv', 'scenario_del_7', 'dataset,model',
             'scenario_del_7_2', 'scenario_del_7_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_lower"] = world.source["resource"]
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_reference"] = world.source["resource"]
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_upper"] = world.source["resource"]
            test_delete.i_delete_source_older_newer_with_resource_types(
                self, resource_types=example["resource_types"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists_by_id(
                self, source_id=self.bigml["source_reference"])

    def test_scenario8(self):
        """
        Scenario: Sucessfully deleting a source in a time range and with a tag:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I store the source id as lower
            And I create a BigML source from file "<data>" with tag "<tag1>" storing results in "<output_dir>"
            And I check that the source exists
            And I store the source id as reference
            And I create a BigML source from file "<data>" with tag "<tag2>" storing results in "<output_dir2>"
            And I check that the source has been created
            And I store the source id as upper
            And I delete the source using --newer-than and --source-tag "<tag1>" storing results in "<output_dir3>"
            Then I check that the reference source doesn't exist
            And I check that the upper source exists
        """
        print(self.test_scenario8.__doc__)
        headers = ["data", "output_dir", "tag1", "tag2", "output_dir2",
                   "output_dir3"]
        examples = [
            ['data/iris.csv', 'scenario_del_8', 'my_tag1', 'my_tag2',
             'scenario_del_8_2', 'scenario_del_8_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_lower"] = world.source["resource"]
            test_delete.i_create_source_from_file_with_tag(
                self, data=example["data"], tag=example["tag1"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)
            self.bigml["source_reference"] = world.source["resource"]
            test_delete.i_create_source_from_file_with_tag(
                self, data=example["data"], tag=example["tag2"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)
            self.bigml["source_upper"] = world.source["resource"]
            test_delete.i_delete_source_newer_and_tag(
                self, tag=example["tag1"], output_dir=example["output_dir"])
            test_delete.i_check_source_does_not_exist(
                self, source_id=self.bigml["source_reference"])
            test_delete.i_check_source_exists_by_id(
                self, source_id=self.bigml["source_upper"])

    def test_scenario9(self):
        """
        Scenario: Sucessfully deleting resources in a time range and with a tag:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I store the source id as lower
            And I create a BigML source from file "<data>" with tag "<tag>" storing results in "<output_dir>"
            And I check that the source exists
            And I create a BigML dataset from the source with tag "<tag>" storing results in "<output_dir2>"
            And I check that the dataset exists
            And I delete the resources using --newer-than and --all-tag "<tag>" storing results in "<output_dir3>"
            Then I check that the source doesn't exist
            And I check that the dataset doesn't exist
        """
        print(self.test_scenario9.__doc__)
        headers = ["data", "output_dir", "tag", "output_dir2", "output_dir3"]
        examples = [
            ['data/iris.csv', 'scenario_del_9', 'my_tag1', 'scenario_del_9_2',
             'scenario_del_9_3']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_lower"] = world.source["resource"]
            test_delete.i_create_source_from_file_with_tag(
                self, data=example["data"], tag=example["tag"],
                output_dir=example["output_dir"])
            test_delete.i_check_source_exists(self)
            self.bigml["source_reference"] = world.source["resource"]
            test_delete.i_create_dataset_from_source_with_tag(
                self, tag=example["tag"], output_dir=example["output_dir"])
            test_delete.i_check_dataset_exists(self)
            test_delete.i_delete_resources_newer_and_tag(
                self, tag=example["tag"], output_dir=example["output_dir"])
            test_delete.i_check_source_does_not_exist(
                self, source_id=self.bigml["source_reference"])
            test_delete.i_check_dataset_does_not_exist(
                self, dataset_id=world.dataset)
            test_delete.i_check_source_exists_by_id(
                self, source_id=self.bigml["source_lower"])

    def test_scenario10(self):
        """
        Scenario: Sucessfully deleting a source in a time range and with a tag and status faulty:
            Given I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source has been created
            And I store the source id as lower
            And I create a BigML source from file "<data>" storing results in "<output_dir>"
            And I check that the source exists
            And I store the source id as reference
            And I create a BigML source from file "<data2>" with tag "<tag2>" storing results in "<output_dir2>"
            And I check that the failed source exists
            And I delete the source using --newer-than and --source-tag "<tag2>" and --status faulty storing results in "<output_dir3>"
            Then I check that the reference source doesn't exist
            And I check that the upper source exists
        """
        print(self.test_scenario10.__doc__)
        headers = ["data", "output_dir", "tag2",
                   "output_dir2", "output_dir3", "data2"]
        examples = [
            ['data/iris.csv', 'scenario_del_10', 'my_tag2',
             'scenario_del_10_2', 'scenario_del_10_3', 'data/faulty.csv']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_source_from_file(
                self, data=example["data"], output_dir=example["output_dir"])
            test_pred.i_check_create_source(self)
            self.bigml["source_lower"] = world.source["resource"]
            test_delete.i_create_faulty_source_from_file_with_tag(
                self, data=example["data2"], tag=example["tag2"],
                output_dir=example["output_dir2"])
            test_delete.i_check_faulty_source_exists(self)
            self.bigml["source_reference"] = world.source["resource"]
            test_delete.i_delete_source_newer_faulty_and_tag(
                self, tag=example["tag2"], output_dir=example["output_dir3"])
            test_delete.i_check_source_does_not_exist(
                self, source_id=self.bigml["source_reference"])
            test_delete.i_check_source_exists_by_id(
                self, source_id=self.bigml["source_lower"])

    def test_scenario11(self):
        """
        Scenario: Successfully deleting a project:
            Given I create a BigML project with name "<name>" storing results in "<output_dir>"
            And I check that the project exists
            And I delete the project by name storing results in "<output_dir>"
            Then I check that the project doesn't exist

            Examples:
            | name               | output_dir       |
            | my_trash_project   | ./scenario_del_11 |
        """
        print(self.test_scenario11.__doc__)
        headers = ["name", "output_dir"]
        examples = [
            ['my_trash_project', 'scenario_del_11']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_delete.i_create_project(
                self, name=example["name"], output_dir=example["output_dir"])
            test_delete.i_check_project_exists(self)
            test_delete.i_delete_project_by_name(self, name=example["name"],
                output_dir=example["output_dir"])
            test_delete.i_check_project_does_not_exist(self)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
#pylint: disable=locally-disabled,line-too-long,attribute-defined-outside-init
#pylint: disable=locally-disabled,too-many-public-methods
#
# Copyright 2015-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


""" Testing reify subcommand

"""
from bigmler.tests.world import (world, common_setup_module,
                                 common_teardown_module, show_method)

import bigmler.tests.reify_steps as test_reify

def setup_module():
    """Setup for the module

    """
    common_setup_module()

def teardown_module():
    """Teardown for the module

    """
    common_teardown_module()

class TestReify:
    """Testing reify command"""

    def setup_method(self, method):
        """
            Debug information
        """
        self.bigml = {}
        self.bigml["method"] = method.__name__
        print("\n-------------------\nTests in: %s\n" % __name__)

    def teardown_method(self):
        """Calling generic teardown for every method

        """
        world.clear_paths()
        print("\nEnd of tests in: %s\n-------------------\n" % __name__)
        self.bigml = {}

    def test_scenario1(self):
        """
        Scenario: Successfully building a source reify output in python
            Given I create a BigML source with data "<data>" and params "<params>"
            And I check that the source has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario1.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re1/reify.py',
             {"name": "my_source_name"},
             'check_files/reify_source.py', 'python'],
            ['data/iris.csv', 'scenario_re1b/reify.ipynb',
             {"name": "my_source_name"},
             'check_files/reify_source.ipynb', 'nb'],
            ['data/iris.csv', 'scenario_re1c/reify.whizzml',
             {"name": "my_source_name"},
             'check_files/reify_source.whizzml', 'whizzml']]

        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_source(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"])
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario2(self):
        """
        Scenario: Successfully building a dataset reify output in python
            Given I create a BigML dataset from a source with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario2.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re2/reify.py',
             {"name": "my_dataset_name"}, 'check_files/reify_dataset.py',
             'python']]

        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset(
                example["data"],
                output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='dataset')
            test_reify.i_check_output_file(self, output=example["output"],
                                           check_file=example["check_file"])

    def test_scenario3(self):
        """
        Scenario: Successfully building a model reify output in python
            Given I create a BigML model from a dataset with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario3.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re3/reify.py',
             {"name": "my_model_name"},
             'check_files/reify_model.py', 'python']]

        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_model(example["data"], output=example["output"],
                                    args=example["params"])
            test_reify.i_create_output(self, example["output"],
                                       example["language"],
                                       resource_type='model')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario4(self):
        """
        Scenario: Successfully building a prediction reify output in python
            Given I create a BigML prediction for "<input_data>" from a model with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the prediction has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario4.__doc__)
        headers = ["data", "output", "params", "check_file", "language",
                   "input_data"]
        examples = [
            ['data/iris.csv', 'scenario_re4/reify.py', {"name": "my_prediction_name"}, 'check_files/reify_prediction.py', 'python', {'petal length': 0.5}]]

        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_prediction(
                example["data"], input_data=example["input_data"],
                output=example["output"], args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='prediction')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario5(self):
        """
        Scenario: Successfully building a cluster reify output in python
            Given I create a BigML cluster from a dataset with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario5.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re5/reify.py',
             {"name": "my_cluster_name"},
             'check_files/reify_cluster.py', 'python']]

        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_cluster(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='cluster')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario6(self):
        """
            Scenario: Successfully building an anomaly detector reify output in python
                Given I create a BigML anomaly from a dataset with data "<data>" and params "<params>"
                And I check that the source has been created
                And I check that the dataset has been created
                And I check that the anomaly detector has been created
                Then I create a reify output in "<output>" for "<language>"
                And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario6.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re6/reify.py',
             {"name": "my_anomaly_name"},
             'check_files/reify_anomaly.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_anomaly(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='anomaly')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario7(self):
        """
        Scenario: Successfully building a centroid reify output in python
            Given I create a BigML centroid for "<input_data>" from a cluster with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the centroid has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario7.__doc__)
        headers = ["data", "output", "params", "check_file", "language",
                   "input_data"]
        examples = [
            ['data/iris.csv', 'scenario_re7/reify.py',
             {"name": "my_centroid_name"}, 'check_files/reify_centroid.py',
             'python', {"petal length": 0.5, "sepal length": 1,
             "petal width": 0.5, "sepal width": 1, "species": "Iris-setosa"}]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_centroid(
                example["data"], input_data=example["input_data"],
                output=example["output"], args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='centroid')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario8(self):
        """
        Scenario: Successfully building an anomaly score reify output in python
            Given I create a BigML anomaly score for "<input_data>" from an anomaly detector with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            And I check that the anomaly score has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario8.__doc__)
        headers = ["data", "output", "params", "check_file", "language",
                   "input_data"]
        examples = [
            ['data/iris.csv', 'scenario_re8/reify.py',
             {"name": "my_anomaly_score_name"},
             'check_files/reify_anomaly_score.py', 'python',
             {"petal length": 0.5, "sepal length": 1, "petal width": 0.5,
              "sepal width": 1, "species": "Iris-setosa"}]]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_anomaly_score(
                example["data"], input_data=example["input_data"],
                output=example["output"], args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='anomaly_score')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario9(self):
        """
        Scenario: Successfully building a batch prediction reify output in python
            Given I create a BigML batch prediction from a model with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the batch prediction has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario9.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re9/reify.py',
             {"name": "my_batch_prediction_name"},
             'check_files/reify_batch_prediction.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_batch_prediction(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_prediction')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario10(self):
        """
        Scenario: Successfully building a batch centroid reify output in python
            Given I create a BigML batch centroid from a cluster with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the batch centroid has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario10.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re10/reify.py',
             {"name": "my_batch_centroid_name"},
             'check_files/reify_batch_centroid.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_batch_centroid(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_centroid')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario11(self):
        """
        Scenario: Successfully building a batch anomaly score reify output in python
            Given I create a BigML batch anomaly score from an anomaly detector with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            And I check that the batch anomaly score has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario11.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re11/reify.py',
             {"name": "my_batch_anomaly_score_name"},
             'check_files/reify_batch_anomaly_score.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_batch_anomaly_score(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_anomaly_score')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario12(self):
        """
        Scenario: Successfully building an evaluation reify output in python
            Given I create a BigML evaluation with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the evaluation has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario12.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re12/reify.py',
             {"name": "my_evaluation_name"},
             'check_files/reify_evaluation.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_evaluation(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='evaluation')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario13(self):
        """
        Scenario: Successfully building an ensemble reify output in python
            Given I create a BigML ensemble from a dataset with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the ensemble has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario13.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re13/reify.py', {"name": "my_ensemble_name"}, 'check_files/reify_ensemble.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_ensemble(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='ensemble')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario14(self):
        """
        Scenario: Successfully building a source reify output with fields in python
            Given I create a BigML source with data "<data>" and params "<params>"
            And I check that the source has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"

            Examples:
            | data | output | params |check_file | language
            | data/iris_sp_chars.csv | scenario_re14/reify.py | {"name": "my_sóurcè_sp_name"}|../check_files/reify_source_sp.py | python
        """
        print(self.test_scenario14.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris_sp_chars.csv', 'scenario_re14/reify.py',
             {"name": "my_sóurcè_sp_name"}, 'check_files/reify_source_sp.py',
             'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_source(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"], add_fields=True)
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario15(self):
        """
        Scenario: Successfully building an evaluation with train/tests split reify output in python
            Given I create a BigML evaluation with data "<data>" and train/test split and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the evaluation has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"

            Examples:
            | data | output | params |check_file | language
            | data/iris.csv | scenario_re15/reify.py | {"name": "my_evaluation_name"}|../check_files/reify_evaluation_split.py | python
        """
        print(self.test_scenario15.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re15/reify.py',
             {"name": "my_evaluation_name"},
             'check_files/reify_evaluation_split.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_evaluation_split(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='evaluation')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario16(self):
        """
        Scenario: Successfully building a dataset reify output in python when built from a batch prediction
            Given I create a BigML dataset from a batch prediction from a model with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the model has been created
            And I check that the batch prediction has been created
            And I check that the batch prediction dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario16.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re16/reify.py',
             {"name": "my_dataset_from_batch_prediction_name"},
             'check_files/reify_batch_prediction_dataset.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_batch_prediction(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_prediction_dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario17(self):
        """
        Scenario: Successfully building a dataset reify output in python when built from a batch centroid
            Given I create a BigML dataset from a batch centroid from a cluster with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the batch centroid has been created
            And I check that the batch centroid dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario17.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re17/reify.py',
             {"name": "my_dataset_from_batch_centroid_name"},
             'check_files/reify_batch_centroid_dataset.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_batch_centroid(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_centroid_dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario18(self):
        """
        Scenario: Successfully building a dataset reify output in python when built from a batch anomaly score
            Given I create a BigML dataset from a batch anomaly score from an anomaly detector with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the anomaly detector has been created
            And I check that the batch anomaly score has been created
            And I check that the batch anomaly score dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"

            Examples:
            | data | output | params |check_file | language
            | data/iris.csv | scenario_re18/reify.py | {"name": "my_dataset_from_batch_anomaly_score_name"}|../check_files/reify_batch_anomaly_score_dataset.py | python
        """
        print(self.test_scenario18.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re18/reify.py',
             {"name": "my_dataset_from_batch_anomaly_score_name"},
             'check_files/reify_batch_anomaly_score_dataset.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_batch_anomaly(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='batch_anomaly_score_dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario19(self):
        """
        Scenario: Successfully building a dataset reify output in python when built from another dataset
            Given I create a BigML dataset from a dataset with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the new dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
       """
        print(self.test_scenario19.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re19/reify.py',
             {"name": "my_dataset_from_dataset_name",
              'all_but': ['000001'], 'all_fields': False,
              'input_fields': ['000000', '000001', '000002', '000003',
                               '000004'],
              'new_fields': [{'description': '', 'field': '2',
                              'label': '', 'name': 'new'}],
              'objective_field': {'id': '100004'}},
              'check_files/reify_dataset_dataset.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_dataset(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario20(self):
        """
        Scenario: Successfully building a dataset reify output in python when built from a dataset from a batch centroid
            Given I create a BigML dataset from a dataset from a batch centroid from a cluster with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            And I check that the cluster has been created
            And I check that the batch centroid has been created
            And I check that the batch centroid dataset has been created
            And I check that the batch centroid dataset dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario20.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re20/reify.py',
             {"name": "my_dataset_from_dataset_from_batch_centroid_name",
              "new_fields": [{"field": "( integer ( replace (" +
              " field \"cluster\" ) \"Cluster \" \"\" ) )",
              "name": "Cluster"}]},
             'check_files/reify_batch_centroid_dataset_dataset.py', 'python']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_dataset_from_batch_centroid(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])

    def test_scenario21(self):
        """
        Scenario: Successfully building a dataset from a list of datasets
        reify output in python
            Given I create a BigML dataset from a list of datasets with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            Then I create a BigML dataset from a source with data "<data>" and params "<params>"
            And I check that the source has been created
            And I check that the dataset has been created
            Then I create a reify output in "<output>" for "<language>"
            And the "<output>" file is like "<check_file>"
        """
        print(self.test_scenario21.__doc__)
        headers = ["data", "output", "params", "check_file", "language"]
        examples = [
            ['data/iris.csv', 'scenario_re21/reify.py',
             {"name": "my_dataset_name"},
             'check_files/reify_dataset_datasets.py', 'python',
             'scenario_re21']]
        for example in examples:
            example = dict(zip(headers, example))
            show_method(self, self.bigml["method"], example)
            test_reify.create_dataset_from_datasets(
                example["data"], output=example["output"],
                args=example["params"])
            test_reify.i_create_output(
                self, example["output"], example["language"],
                resource_type='dataset')
            test_reify.i_check_output_file(
                self, output=example["output"],
                check_file=example["check_file"])



# -*- coding: utf-8 -*-
#
# Copyright 2019-2025 BigML
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""BigMLer - Resources processing: creation, update and retrieval
of Fusions

"""


from bigml.fields import Fields, DEFAULT_MISSING_TOKENS

import bigmler.utils as u
import bigmler.resourcesapi.fusions as r
import bigmler.checkpoint as c


def has_fusion(args):
    """Returns whether some kind of Fusion reference is given

    """
    return args.fusion or args.fusions or \
        args.fusion_tag


def fusion_processing(fusion, \
    fusion_ids, api, args, resume, fields=None, \
    session_file=None, path=None, log=None):
    """Creates or retrieves fusion from the input data

    """

    # If we have a models' list but not a fusion,
    # we create the model if the no_model
    # flag hasn't been set up.
    if args.fusion_models_ is not None and not has_fusion(args):
        fusion_ids = []

        # Only 1 fusion per bigmler command at present
        number_of_fusions = 1
        if resume:
            resume, fusion_ids = c.checkpoint( \
                c.are_fusions_created, path, \
                number_of_fusions, debug=args.debug)
            if not resume:
                message = u.dated("Found %s fusions out of %s."
                                  " Resuming.\n"
                                  % (len(fusion_ids),
                                     number_of_fusions))
                u.log_message(message, log_file=session_file,
                              console=args.verbosity)

            fusion = fusion_ids[0]
            first_model_id = api.get_fusion(fusion)[ \
                "object"]["fusion"]["models"][0]["id"]
            first_model_kind = api.get_fusion(fusion)[ \
                "object"]["fusion"]["models"][0]["kind"]
            first_model = api.getters[first_model_kind](first_model_id)
            fields = Fields(first_model)
            number_of_fusions -= len(fusion_ids)

        fusion_args = r.set_fusion_args( \
            args, fields)
        fusion = \
            r.create_fusion( \
            args.fusion_models_, fusion, fusion_args, \
            args, api, path, session_file, log)

    # If a fusion is provided, we use it.
    elif args.fusion:
        fusion_ids = [args.fusion]
        fusion = fusion_ids[0]

    elif args.fusion or args.fusion_tag:
        fusion = fusion_ids[0]

    # If we are going to create predictions, we must retrieve the fusion
    if fusion_ids and args.test_set:
        fusion = \
            r.get_fusion(fusion, args, api, session_file)
        args.objective_field = fusion['object']['objective_field_name']

    return fusion, resume


def get_fusion_fields(fusion, csv_properties, args):
    """Retrieves fields info from Fusion resource

    """
    args.retrieve_api_.ok(fusion)
    if not csv_properties:
        csv_properties = {}
    csv_properties.update(verbose=True)
    csv_properties.update(missing_tokens=DEFAULT_MISSING_TOKENS)
    return Fields(fusion['object']['fusion']['fields'], \
        **csv_properties)
